;**** A P P L I C A T I O N   N O T E **************************************
;*
;* Title:               Divide Routines - Speed Optimized
;* Version:             1.1
;* Last Updated:        October 15, 2001
;* Target:              AT94K05/10/40
;*
;* Support Hotline:     +1 (408) 436-4119
;* Support Email:       fpslic@atmel.com
;*
;***************************************************************************

.nolist
.include "at94kdef.inc"
.list

        jmp     RESET


;***************************************************************************
;*
;* "div8u" - 8/8 Bit Unsigned Division
;*
;* This subroutine divides the two register variables "dd8u" (dividend) and 
;* "dv8u" (divisor). The result is placed in "dres8u" and the remainder in
;* "drem8u".
;*  
;* Number of words      :66 + return
;* Number of cycles     :50/58/66 (Min/Avg/Max) + return
;* Low registers used   :1 (drem8u)
;* High registers used  :2 (dres8u/dd8u,dv8u)
;*
;***************************************************************************

;***** Subroutine Register Variables

.def    drem8u  =r15            ;remainder
.def    dres8u  =r16            ;result
.def    dd8u    =r16            ;dividend
.def    dv8u    =r17            ;divisor

;***** Code

div8u:  sub     drem8u,drem8u   ;clear remainder and carry
        
        rol     dd8u            ;shift left dividend
        rol     drem8u          ;shift dividend into remainder
        sub     drem8u,dv8u     ;remainder = remainder - divisor
        brcc    d8u_1           ;if result negative
        add     drem8u,dv8u     ;    restore remainder
        clc                     ;    clear carry to be shifted into result
        rjmp    d8u_2           ;else
d8u_1:  sec                     ;    set carry to be shifted into result

d8u_2:  rol     dd8u            ;shift left dividend
        rol     drem8u          ;shift dividend into remainder
        sub     drem8u,dv8u     ;remainder = remainder - divisor
        brcc    d8u_3           ;if result negative
        add     drem8u,dv8u     ;    restore remainder
        clc                     ;    clear carry to be shifted into result
        rjmp    d8u_4           ;else
d8u_3:  sec                     ;    set carry to be shifted into result

d8u_4:  rol     dd8u            ;shift left dividend
        rol     drem8u          ;shift dividend into remainder
        sub     drem8u,dv8u     ;remainder = remainder - divisor
        brcc    d8u_5           ;if result negative
        add     drem8u,dv8u     ;    restore remainder
        clc                     ;    clear carry to be shifted into result
        rjmp    d8u_6           ;else
d8u_5:  sec                     ;    set carry to be shifted into result

d8u_6:  rol     dd8u            ;shift left dividend
        rol     drem8u          ;shift dividend into remainder
        sub     drem8u,dv8u     ;remainder = remainder - divisor
        brcc    d8u_7           ;if result negative
        add     drem8u,dv8u     ;    restore remainder
        clc                     ;    clear carry to be shifted into result
        rjmp    d8u_8           ;else
d8u_7:  sec                     ;    set carry to be shifted into result

d8u_8:  rol     dd8u            ;shift left dividend
        rol     drem8u          ;shift dividend into remainder
        sub     drem8u,dv8u     ;remainder = remainder - divisor
        brcc    d8u_9           ;if result negative
        add     drem8u,dv8u     ;    restore remainder
        clc                     ;    clear carry to be shifted into result
        rjmp    d8u_10          ;else
d8u_9:  sec                     ;    set carry to be shifted into result

d8u_10: rol     dd8u            ;shift left dividend
        rol     drem8u          ;shift dividend into remainder
        sub     drem8u,dv8u     ;remainder = remainder - divisor
        brcc    d8u_11          ;if result negative
        add     drem8u,dv8u     ;    restore remainder
        clc                     ;    clear carry to be shifted into result
        rjmp    d8u_12          ;else
d8u_11: sec                     ;    set carry to be shifted into result

d8u_12: rol     dd8u            ;shift left dividend
        rol     drem8u          ;shift dividend into remainder
        sub     drem8u,dv8u     ;remainder = remainder - divisor
        brcc    d8u_13          ;if result negative
        add     drem8u,dv8u     ;    restore remainder
        clc                     ;    clear carry to be shifted into result
        rjmp    d8u_14          ;else
d8u_13: sec                     ;    set carry to be shifted into result

d8u_14: rol     dd8u            ;shift left dividend
        rol     drem8u          ;shift dividend into remainder
        sub     drem8u,dv8u     ;remainder = remainder - divisor
        brcc    d8u_15          ;if result negative
        add     drem8u,dv8u     ;    restore remainder
        clc                     ;    clear carry to be shifted into result
        rjmp    d8u_16          ;else
d8u_15: sec                     ;    set carry to be shifted into result

d8u_16: rol     dd8u            ;shift left dividend
        ret


;***************************************************************************
;*
;* "div16u" - 16/16 Bit Unsigned Division
;*
;* This subroutine divides the two 16-bit numbers 
;* "dd8uH:dd8uL" (dividend) and "dv16uH:dv16uL" (divisor). 
;* The result is placed in "dres16uH:dres16uL" and the remainder in
;* "drem16uH:drem16uL".
;*  
;* Number of words      :196 + return
;* Number of cycles     :148/173/196 (Min/Avg/Max)
;* Low registers used   :2 (drem16uL,drem16uH)
;* High registers used  :4 (dres16uL/dd16uL,dres16uH/dd16uH,dv16uL,dv16uH)
;*
;***************************************************************************

;***** Subroutine Register Variables

.def    drem16uL=r14
.def    drem16uH=r15
.def    dres16uL=r16
.def    dres16uH=r17
.def    dd16uL  =r16
.def    dd16uH  =r17
.def    dv16uL  =r18
.def    dv16uH  =r19

;***** Code

div16u: clr     drem16uL        ;clear remainder Low byte
        sub     drem16uH,drem16uH;clear remainder High byte and carry

        rol     dd16uL          ;shift left dividend
        rol     dd16uH
        rol     drem16uL        ;shift dividend into remainder
        rol     drem16uH
        sub     drem16uL,dv16uL ;remainder = remainder - divisor
        sbc     drem16uH,dv16uH ;
        brcc    d16u_1          ;if result negative
        add     drem16uL,dv16uL ;    restore remainder
        adc     drem16uH,dv16uH
        clc                     ;    clear carry to be shifted into result
        rjmp    d16u_2          ;else
d16u_1: sec                     ;    set carry to be shifted into result

d16u_2: rol     dd16uL          ;shift left dividend
        rol     dd16uH
        rol     drem16uL        ;shift dividend into remainder
        rol     drem16uH
        sub     drem16uL,dv16uL ;remainder = remainder - divisor
        sbc     drem16uH,dv16uH ;
        brcc    d16u_3          ;if result negative
        add     drem16uL,dv16uL ;    restore remainder
        adc     drem16uH,dv16uH
        clc                     ;    clear carry to be shifted into result
        rjmp    d16u_4          ;else
d16u_3: sec                     ;    set carry to be shifted into result

d16u_4: rol     dd16uL          ;shift left dividend
        rol     dd16uH
        rol     drem16uL        ;shift dividend into remainder
        rol     drem16uH
        sub     drem16uL,dv16uL ;remainder = remainder - divisor
        sbc     drem16uH,dv16uH ;
        brcc    d16u_5          ;if result negative
        add     drem16uL,dv16uL ;    restore remainder
        adc     drem16uH,dv16uH
        clc                     ;    clear carry to be shifted into result
        rjmp    d16u_6          ;else
d16u_5: sec                     ;    set carry to be shifted into result

d16u_6: rol     dd16uL          ;shift left dividend
        rol     dd16uH
        rol     drem16uL        ;shift dividend into remainder
        rol     drem16uH
        sub     drem16uL,dv16uL ;remainder = remainder - divisor
        sbc     drem16uH,dv16uH ;
        brcc    d16u_7          ;if result negative
        add     drem16uL,dv16uL ;    restore remainder
        adc     drem16uH,dv16uH
        clc                     ;    clear carry to be shifted into result
        rjmp    d16u_8          ;else
d16u_7: sec                     ;    set carry to be shifted into result

d16u_8: rol     dd16uL          ;shift left dividend
        rol     dd16uH
        rol     drem16uL        ;shift dividend into remainder
        rol     drem16uH
        sub     drem16uL,dv16uL ;remainder = remainder - divisor
        sbc     drem16uH,dv16uH ;
        brcc    d16u_9          ;if result negative
        add     drem16uL,dv16uL ;    restore remainder
        adc     drem16uH,dv16uH
        clc                     ;    clear carry to be shifted into result
        rjmp    d16u_10         ;else
d16u_9: sec                     ;    set carry to be shifted into result

d16u_10:rol     dd16uL          ;shift left dividend
        rol     dd16uH
        rol     drem16uL        ;shift dividend into remainder
        rol     drem16uH
        sub     drem16uL,dv16uL ;remainder = remainder - divisor
        sbc     drem16uH,dv16uH ;
        brcc    d16u_11         ;if result negative
        add     drem16uL,dv16uL ;    restore remainder
        adc     drem16uH,dv16uH
        clc                     ;    clear carry to be shifted into result
        rjmp    d16u_12         ;else
d16u_11:sec                     ;    set carry to be shifted into result

d16u_12:rol     dd16uL          ;shift left dividend
        rol     dd16uH
        rol     drem16uL        ;shift dividend into remainder
        rol     drem16uH
        sub     drem16uL,dv16uL ;remainder = remainder - divisor
        sbc     drem16uH,dv16uH ;
        brcc    d16u_13         ;if result negative
        add     drem16uL,dv16uL ;    restore remainder
        adc     drem16uH,dv16uH
        clc                     ;    clear carry to be shifted into result
        rjmp    d16u_14         ;else
d16u_13:sec                     ;    set carry to be shifted into result

d16u_14:rol     dd16uL          ;shift left dividend
        rol     dd16uH
        rol     drem16uL        ;shift dividend into remainder
        rol     drem16uH
        sub     drem16uL,dv16uL ;remainder = remainder - divisor
        sbc     drem16uH,dv16uH ;
        brcc    d16u_15         ;if result negative
        add     drem16uL,dv16uL ;    restore remainder
        adc     drem16uH,dv16uH
        clc                     ;    clear carry to be shifted into result
        rjmp    d16u_16         ;else
d16u_15:sec                     ;    set carry to be shifted into result

d16u_16:rol     dd16uL          ;shift left dividend
        rol     dd16uH
        rol     drem16uL        ;shift dividend into remainder
        rol     drem16uH
        sub     drem16uL,dv16uL ;remainder = remainder - divisor
        sbc     drem16uH,dv16uH ;
        brcc    d16u_17         ;if result negative
        add     drem16uL,dv16uL ;    restore remainder
        adc     drem16uH,dv16uH
        clc                     ;    clear carry to be shifted into result
        rjmp    d16u_18         ;else
d16u_17:        sec                     ;    set carry to be shifted into result

d16u_18:rol     dd16uL          ;shift left dividend
        rol     dd16uH
        rol     drem16uL        ;shift dividend into remainder
        rol     drem16uH
        sub     drem16uL,dv16uL ;remainder = remainder - divisor
        sbc     drem16uH,dv16uH ;
        brcc    d16u_19         ;if result negative
        add     drem16uL,dv16uL ;    restore remainder
        adc     drem16uH,dv16uH
        clc                     ;    clear carry to be shifted into result
        rjmp    d16u_20         ;else
d16u_19:sec                     ;    set carry to be shifted into result

d16u_20:rol     dd16uL          ;shift left dividend
        rol     dd16uH
        rol     drem16uL        ;shift dividend into remainder
        rol     drem16uH
        sub     drem16uL,dv16uL ;remainder = remainder - divisor
        sbc     drem16uH,dv16uH ;
        brcc    d16u_21         ;if result negative
        add     drem16uL,dv16uL ;    restore remainder
        adc     drem16uH,dv16uH
        clc                     ;    clear carry to be shifted into result
        rjmp    d16u_22         ;else
d16u_21:sec                     ;    set carry to be shifted into result

d16u_22:rol     dd16uL          ;shift left dividend
        rol     dd16uH
        rol     drem16uL        ;shift dividend into remainder
        rol     drem16uH
        sub     drem16uL,dv16uL ;remainder = remainder - divisor
        sbc     drem16uH,dv16uH ;
        brcc    d16u_23         ;if result negative
        add     drem16uL,dv16uL ;    restore remainder
        adc     drem16uH,dv16uH
        clc                     ;    clear carry to be shifted into result
        rjmp    d16u_24         ;else
d16u_23:sec                     ;    set carry to be shifted into result

d16u_24:rol     dd16uL          ;shift left dividend
        rol     dd16uH
        rol     drem16uL        ;shift dividend into remainder
        rol     drem16uH
        sub     drem16uL,dv16uL ;remainder = remainder - divisor
        sbc     drem16uH,dv16uH ;
        brcc    d16u_25         ;if result negative
        add     drem16uL,dv16uL ;    restore remainder
        adc     drem16uH,dv16uH
        clc                     ;    clear carry to be shifted into result
        rjmp    d16u_26         ;else
d16u_25:sec                     ;    set carry to be shifted into result

d16u_26:rol     dd16uL          ;shift left dividend
        rol     dd16uH
        rol     drem16uL        ;shift dividend into remainder
        rol     drem16uH
        sub     drem16uL,dv16uL ;remainder = remainder - divisor
        sbc     drem16uH,dv16uH ;
        brcc    d16u_27         ;if result negative
        add     drem16uL,dv16uL ;    restore remainder
        adc     drem16uH,dv16uH
        clc                     ;    clear carry to be shifted into result
        rjmp    d16u_28         ;else
d16u_27:sec                     ;    set carry to be shifted into result

d16u_28:rol     dd16uL          ;shift left dividend
        rol     dd16uH
        rol     drem16uL        ;shift dividend into remainder
        rol     drem16uH
        sub     drem16uL,dv16uL ;remainder = remainder - divisor
        sbc     drem16uH,dv16uH ;
        brcc    d16u_29         ;if result negative
        add     drem16uL,dv16uL ;    restore remainder
        adc     drem16uH,dv16uH
        clc                     ;    clear carry to be shifted into result
        rjmp    d16u_30         ;else
d16u_29:sec                     ;    set carry to be shifted into result

d16u_30:rol     dd16uL          ;shift left dividend
        rol     dd16uH
        rol     drem16uL        ;shift dividend into remainder
        rol     drem16uH
        sub     drem16uL,dv16uL ;remainder = remainder - divisor
        sbc     drem16uH,dv16uH ;
        brcc    d16u_31         ;if result negative
        add     drem16uL,dv16uL ;    restore remainder
        adc     drem16uH,dv16uH
        clc                     ;    clear carry to be shifted into result
        rjmp    d16u_32         ;else
d16u_31:sec                     ;    set carry to be shifted into result

d16u_32:rol     dd16uL          ;shift left dividend
        rol     dd16uH
        ret



;****************************************************************************
;*
;* Test Program
;*
;* This program calls all the subroutines as an example of usage and to 
;* verify correct verification.
;*
;****************************************************************************

;***** Main Program Register variables

.def    temp    =r16            ;temporary storage variable

;***** Code
RESET:
        ldi     temp, high($0FFF)
        out     SPH, temp
        ldi     temp, low($0FFF)
        out     SPL, temp       ;setup stack pointer

;***** Divide Two Unsigned 8-Bit Numbers (100/3)
        ldi     dd8u,100
        ldi     dv8u,3
        rcall   div8u           ;result:        $21 (33)
                                ;remainder:     $01 (1)

;***** Divide Two Unsigned 16-Bit Numbers (50,000/24,995)
        ldi     dd16uL,low(50000)
        ldi     dd16uH,high(50000)
        ldi     dv16uL,low(24995)
        ldi     dv16uH,high(24995)
        rcall   div16u          ;result:        $0002 (2)
                                ;remainder:     $000a (10)

forever: rjmp    forever
