library IEEE;
use IEEE.STD_Logic_1164.all, IEEE.Numeric_STD.all;

library work;
use work.definitions.all;

-- ************************************************************************************************************************************************ --
entity arbiter_interface is
	port (	Clock, Reset	: in std_logic;
		
		Dividend_1,
		Dividend_2,
		Dividend_3,
		Dividend_4,
		Dividend_5	: in std_logic_vector(data_width*2 - 1 downto 0);
		
		Divisor_1,
		Divisor_2,
		Divisor_3,
		Divisor_4,
		Divisor_5	: in std_logic_vector(data_width -1 downto 0);
		
		Req_1,
		Req_2,
		Req_3,
		Req_4,
		Req_5		: in std_logic;
		
		Ack_1,
		Ack_2,
		Ack_3,
		Ack_4,
		Ack_5		: out std_logic;
		
		Result_1,
		Result_2,
		Result_3,
		Result_4,
		Result_5	: out std_logic_vector(speed_width -1 downto 0);
		
		Dividend_out	: out std_logic_vector(data_width*2 - 1 downto 0);
		Divisor_out	: out std_logic_vector(data_width   - 1 downto 0);
		Result_in	: in  std_logic_vector(data_width*2 downto 0)		-- forelpig ikke implementert. gr direkte.
		
	);
end entity arbiter_interface;
-- ************************************************************************************************************************************************ --


-- ************************************************************************************************************************************************ --
architecture division_arbiter of arbiter_interface is

	type ARB_STATE_TYPE is (Idle,Grant_1,Grant_2,Grant_3, Grant_4, Grant_5);	
	signal CurrentState, NextState, LastState: ARB_STATE_TYPE;
	signal En11, En12, En21, En22, En31, En32, En41, En42, En51, En52 : std_logic;

begin

-- *********************************************************** --
-- *********************************************************** --
answer_ready_poller : process(Result_in) is

	variable last_result 	: std_logic_vector(speed_width-1 downto 0) := (others => '0');
	
	alias divide_by_zero 	is Result_in(data_width*2);
	alias result 		is Result_in(data_width+speed_width-1 downto data_width);
	alias remainder 	is Result_in(data_width*1-1 downto 0);
begin
	----------------------------------------------------------
	if (result /= last_result) and (divide_by_zero = '0') then
		--------------------------------------------------
		case LastState is
			when Grant_1 => Result_1 <= result;
			when Grant_2 => Result_2 <= result;
			when Grant_3 => Result_3 <= result;
			when Grant_4 => Result_4 <= result;
			when Grant_5 => Result_5 <= result;
			when others => null;
		end case;
		--------------------------------------------------
		last_result := result;--current_result;
	----------------------------------------------------------
	elsif (divide_by_zero = '1') then				-- Gjr systemet bedre, men gjr ogs simuleringene vanskeligere  lese.
		Result_1 <= (others => '0');
		Result_2 <= (others => '0');
		Result_3 <= (others => '0');
		Result_4 <= (others => '0');
		Result_5 <= (others => '0');
	----------------------------------------------------------
	end if;
	
end process;
-- *********************************************************** --


-- *********************************************************** --
-- *********************************************************** --
ARBITER_COMB : process(Req_1, Req_2, Req_3, Req_4, Req_5,CurrentState)

begin
	Ack_1 <= '0'; Ack_2 <= '0'; Ack_3 <= '0'; Ack_4 <= '0';	Ack_5 <= '0';

	case CurrentState is
		-- Her kunne jeg fint bygget inn noe litt mer rettferdig. Men tar meg ikke tid n.
		-----------------------------------------------------
		when Idle =>
			---------------------------------------------
			if	Req_1 = '1' then NextState <= Grant_1;
			elsif 	Req_2 = '1' then NextState <= Grant_2;
			elsif 	Req_3 = '1' then NextState <= Grant_3;
			elsif 	Req_4 = '1' then NextState <= Grant_4;
			elsif 	Req_5 = '1' then NextState <= Grant_5;
			else			 NextState <= Idle;
			end if;
		-----------------------------------------------------
		when Grant_1 =>	Ack_1 <= '1';
			---------------------------------------------
			if 	Req_2 = '1' then NextState <= Grant_2;
			elsif 	Req_3 = '1' then NextState <= Grant_3;
			elsif 	Req_4 = '1' then NextState <= Grant_4;
			elsif 	Req_5 = '1' then NextState <= Grant_5;
			elsif 	Req_1 = '1' then NextState <= Grant_1;
			else 			 NextState <= Idle;
			end if;
		-----------------------------------------------------
		when Grant_2 =>	Ack_2 <= '1';
			---------------------------------------------
			if 	Req_3 = '1' then NextState <= Grant_3;
			elsif 	Req_4 = '1' then NextState <= Grant_4;
			elsif 	Req_5 = '1' then NextState <= Grant_5;
			elsif 	Req_1 = '1' then NextState <= Grant_1;
			elsif 	Req_2 = '1' then NextState <= Grant_2;
			else 			 NextState <= Idle;
			end if;
		-----------------------------------------------------
		when Grant_3 =>	Ack_3 <= '1';
			---------------------------------------------
			if 	Req_4 = '1' then NextState <= Grant_4;
			elsif 	Req_5 = '1' then NextState <= Grant_5;
			elsif 	Req_1 = '1' then NextState <= Grant_1;
			elsif 	Req_2 = '1' then NextState <= Grant_2;
			elsif 	Req_3 = '1' then NextState <= Grant_3;
			else 			 NextState <= Idle;
			end if;
		-----------------------------------------------------
		when Grant_4 => Ack_4 <= '1';
			---------------------------------------------
			if 	Req_5 = '1' then NextState <= Grant_5;
			elsif 	Req_1 = '1' then NextState <= Grant_1;
			elsif 	Req_2 = '1' then NextState <= Grant_2;
			elsif 	Req_3 = '1' then NextState <= Grant_3;
			elsif 	Req_4 = '1' then NextState <= Grant_4;
			else 			 NextState <= Idle;
			end if;
		-----------------------------------------------------
		when Grant_5 => Ack_5 <= '1';
			---------------------------------------------
			if 	Req_1 = '1' then NextState <= Grant_1;
			elsif 	Req_2 = '1' then NextState <= Grant_2;
			elsif 	Req_3 = '1' then NextState <= Grant_3;
			elsif 	Req_4 = '1' then NextState <= Grant_4;
			elsif 	Req_5 = '1' then NextState <= Grant_5;
			else 			 NextState <= Idle;
			end if;
		-----------------------------------------------------
		when others => null;
	end case;

end process;
-- *********************************************************** --


-- *********************************************************** --
-- *********************************************************** --
ARBITER_SEQ : process(Reset, Clock)

	variable temp_state : ARB_STATE_TYPE;
begin
	if(Reset = '1') then
		CurrentState <= Idle;
		LastState <= Idle;
		temp_state := Idle;	-- Added. Smart?
	elsif rising_edge(Clock) then
		LastState <= temp_state;
		temp_state := CurrentState;
		Currentstate <= NextState;
	end if;	
end process;
-- *********************************************************** --


-- *********************************************************** --
-- *********************************************************** --
SYNC_TRI_STATE_ENS : process(Reset, Clock)
begin
	if 	(Reset = '1') then

		En11 <= '0'; En12 <= '0';	
		En21 <= '0'; En22 <= '0';	
		En31 <= '0'; En32 <= '0';
		En41 <= '0'; En42 <= '0';	
		En51 <= '0'; En52 <= '0';

	elsif rising_edge(Clock) then

		En11 <= '0'; En12 <= '0';
		En21 <= '0'; En22 <= '0';
		En31 <= '0'; En32 <= '0';
		En41 <= '0'; En42 <= '0';
		En51 <= '0'; En52 <= '0';
		
		case NextState is
			when Grant_1 => En11 <= '1'; En12 <= '1';
			when Grant_2 => En21 <= '1'; En22 <= '1';
			when Grant_3 => En31 <= '1'; En32 <= '1';
			when Grant_4 => En41 <= '1'; En42 <= '1';
			when Grant_5 => En51 <= '1'; En52 <= '1';
			when others =>
				En11 <= '0'; En12 <= '0';
				En21 <= '0'; En22 <= '0';
				En31 <= '0'; En32 <= '0';
				En41 <= '0'; En42 <= '0';
				En51 <= '0'; En52 <= '0';
		end case;
	end if;
end process;
-- *********************************************************** --


-- *********************************************************** --
-- *********************************************************** --
Dividend_out <= Dividend_1 when (En11 = '1') else (others => 'Z');
Dividend_out <= Dividend_2 when (En21 = '1') else (others => 'Z');
Dividend_out <= Dividend_3 when (En31 = '1') else (others => 'Z');
Dividend_out <= Dividend_4 when (En41 = '1') else (others => 'Z');
Dividend_out <= Dividend_5 when (En51 = '1') else (others => 'Z');

-- Gjr systemet bedre. Men blir vanskelig  lese simuleringene. Fjernes midlertidig	
--Dividend_out <= (others => '0') when (En11 = '0' and En21 = '0' and En31 = '0' and En41 = '0' and En51 = '0') else (others => 'Z');
-- En bedre variant, som virker fra oppstart av.
Dividend_out <= (others => '0') when (En11 /= '1' and En21 /= '1' and En31 /= '1' and En41 /= '1' and En51 /= '1') else (others => 'Z');
	
Divisor_out <= Divisor_1 when (En12 = '1') else (others => 'Z');
Divisor_out <= Divisor_2 when (En22 = '1') else (others => 'Z');
Divisor_out <= Divisor_3 when (En32 = '1') else (others => 'Z');
Divisor_out <= Divisor_4 when (En42 = '1') else (others => 'Z');
Divisor_out <= Divisor_5 when (En52 = '1') else (others => 'Z');

-- Gjr systemet bedre. Men blir vanskelig  lese simuleringene. Fjernes midlertidig	
--Divisor_out <= (others => '0') when (En12 = '0' and En22 = '0' and En32 = '0' and En42 = '0' and En52 = '0') else (others => 'Z');
-- En bedre variant, som virker fra oppstart av.
Divisor_out <= (others => '0') when (En12 /= '1' and En22 /= '1' and En32 /= '1' and En42 /= '1' and En52 /= '1') else (others => 'Z');
-- *********************************************************** --

end architecture;
-- ************************************************************************************************************************************************ --