library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

library work;
use work.definitions.all;

-- ****************************************************************************************************************** --
entity motor_communication_interface is

port(	-- ***** System interface ***** --
	clk		: in std_logic;		-- Br nok kjre p samme klokke som AVR.
	reset		: in std_logic;

	-- ***** Calling interface ***** --
	IOselect	: in 	std_logic_vector(4 downto 0);	-- Interrupt som iverksetter utveksling av data. Throttle inn og Speed ut.
	IOwrite		: in 	std_logic;			-- Gir FPGA signal om at AVR er klar for mottak av data.
	IOread		: in 	std_logic;			-- Gir FPGA signal om at data blir sendt fra AVR
	data_bus	: inout std_logic_vector(7 downto 0); 	-- Felles data-buss for sending og mottak av data.
	Interrupt_out	: out	std_logic_vector(4 downto 0);	-- Velger forelpig  ha dedikerte interrupt.
	
	-- ***** Motor interface ***** --
	m1_direction	: out	std_logic;
	m1_throttle	: out	std_logic_vector(7 downto 0);
	m1_current_speed: in	std_logic_vector(speed_width-1 downto 0);
	
	m2_direction	: out	std_logic;
	m2_throttle	: out	std_logic_vector(7 downto 0);
	m2_current_speed: in	std_logic_vector(speed_width-1 downto 0);
	
	m3_direction	: out	std_logic;
	m3_throttle	: out	std_logic_vector(7 downto 0);
	m3_current_speed: in	std_logic_vector(speed_width-1 downto 0);
	
	m4_direction	: out	std_logic;
	m4_throttle	: out	std_logic_vector(7 downto 0);
	m4_current_speed: in	std_logic_vector(speed_width-1 downto 0);
	
	m5_direction	: out	std_logic;
	m5_throttle	: out	std_logic_vector(7 downto 0);
	m5_current_speed: in	std_logic_vector(speed_width-1 downto 0)
	
);

end entity motor_communication_interface;
-- ****************************************************************************************************************** --


-- ****************************************************************************************************************** --
architecture motor_communicatior of motor_communication_interface is

	signal motor_interrupt	 : std_logic_vector(4 downto 0) := (others => '0');
	signal interrupt_handeled: std_logic_vector(4 downto 0) := "00000";		-- Et eksperiment
	signal cancel_interrupt	 : std_logic_vector(4 downto 0) := (others => '0');
	
	signal m1_low_byte_sent	 : std_logic := '0';	-- Debug-signal
	signal m2_low_byte_sent	 : std_logic := '0';	-- Debug-signal
	signal m3_low_byte_sent	 : std_logic := '0';	-- Debug-signal
	signal m4_low_byte_sent	 : std_logic := '0';	-- Debug-signal
	signal m5_low_byte_sent	 : std_logic := '0';	-- Debug-signal
	
	-- De flgende blir ikke resat. Trengs det?
	signal m1_current_speed_int : std_logic_vector(speed_width-1 downto 0) := (others => '0');
	signal m2_current_speed_int : std_logic_vector(speed_width-1 downto 0) := (others => '0');
	signal m3_current_speed_int : std_logic_vector(speed_width-1 downto 0) := (others => '0');
	signal m4_current_speed_int : std_logic_vector(speed_width-1 downto 0) := (others => '0');
	signal m5_current_speed_int : std_logic_vector(speed_width-1 downto 0) := (others => '0');
	
	signal m1_data_out : std_logic_vector(7 downto 0) := (others => '0');
	signal m2_data_out : std_logic_vector(7 downto 0) := (others => '0');
	signal m3_data_out : std_logic_vector(7 downto 0) := (others => '0');
	signal m4_data_out : std_logic_vector(7 downto 0) := (others => '0');
	signal m5_data_out : std_logic_vector(7 downto 0) := (others => '0');
	
	signal m1_write_enable 	: std_logic := '0';
	signal m2_write_enable 	: std_logic := '0';
	signal m3_write_enable 	: std_logic := '0';
	signal m4_write_enable 	: std_logic := '0';
	signal m5_write_enable 	: std_logic := '0';
	
	signal speed_changed 	: std_logic_vector (4 downto 0) := "00000";
	signal interrupt_set 	: std_logic_vector (4 downto 0) := "00000";
	signal interrupt_reset	: std_logic_vector (4 downto 0) := "00000";
	
	signal 	m1_last_speed, 
		m2_last_speed, 
		m3_last_speed, 
		m4_last_speed, 
		m5_last_speed : std_logic_vector(speed_width-1 downto 0) := (others => '0');
	
	
begin


-- *************************************************************************************** --
-- *	Sjekker om hastighetene er endret. Sender beskjed hvis ja.
-- *	Kan ikke sette interrupt selv, siden de da ikke ville foreligge samtidig som data ut.
-- *************************************************************************************** --
new_speed_poller : process(	m1_current_speed_int,
				m2_current_speed_int, 
				m3_current_speed_int, 
				m4_current_speed_int, 
				m5_current_speed_int,
				m1_last_speed,
				m2_last_speed,
				m3_last_speed,
				m4_last_speed,
				m5_last_speed) is
	
begin
	--speed_changed <= '0';
	-----------------------------------------------------------------------------------------------
	if 	m1_current_speed_int /= m1_last_speed then speed_changed <= (m1 => '1', others => '0');	
	-----------------------------------------------------------------------------------------------	
	elsif 	m2_current_speed_int /= m2_last_speed then speed_changed <= (m2 => '1', others => '0');
	-----------------------------------------------------------------------------------------------
	elsif 	m3_current_speed_int /= m3_last_speed then speed_changed <= (m3 => '1', others => '0');
	-----------------------------------------------------------------------------------------------
	elsif 	m4_current_speed_int /= m4_last_speed then speed_changed <= (m4 => '1', others => '0');
	-----------------------------------------------------------------------------------------------
	elsif 	m5_current_speed_int /= m5_last_speed then speed_changed <= (m5 => '1', others => '0');
	-----------------------------------------------------------------------------------------------
	else	speed_changed <= (others => '0');
	-----------------------------------------------------------------------------------------------
	end if;

end process;
-- *************************************************************************************** --


-- *************************************************************************************** --
-- *
-- *************************************************************************************** --
interrupt_setter : process (clk)--, reset) is	-- Added reset. Okei?


begin	
	
	----------------------------------------------------------
	if rising_edge (clk) then
		--------------------------------------------------
--		if 	reset = '1' then
--			m1_last_speed <= (others => '0');
--			m2_last_speed <= (others => '0');
--			m3_last_speed <= (others => '0');
--			m4_last_speed <= (others => '0');
--			m5_last_speed <= (others => '0');
--			interrupt_set <= (others => '0');	-- Added for testing.
		--------------------------------------------------
		--else
			interrupt_set <= (others => '0');
			
			------------------------------------------
			if 	speed_changed(m1) = '1' then
				----------------------------------
				m1_last_speed <= m1_current_speed;
				interrupt_set(m1) <= '1';
			------------------------------------------
			elsif 	speed_changed(m2) = '1' then
				----------------------------------
				m2_last_speed <= m2_current_speed;
				interrupt_set(m2) <= '1';
			------------------------------------------
			elsif 	speed_changed(m3) = '1' then
				----------------------------------
				m3_last_speed <= m3_current_speed;
				interrupt_set(m3) <= '1';
			------------------------------------------
			elsif 	speed_changed(m4) = '1' then
				----------------------------------
				m4_last_speed <= m4_current_speed;
				interrupt_set(m4) <= '1';
			------------------------------------------
			elsif 	speed_changed(m5) = '1' then
				----------------------------------
				m5_last_speed <= m5_current_speed;
				interrupt_set(m5) <= '1';
			------------------------------------------
			end if;
		--------------------------------------------------
		--end if;
	----------------------------------------------------------
	end if;
end process;
-- *************************************************************************************** --



-- *************************************************************************************** --
-- *
-- *************************************************************************************** --
sr_latch : process (interrupt_set, interrupt_handeled)

	variable v_srlatch_intern	: std_logic_vector(4 downto 0)	:= "00000"; -- Tester  initialisere!
	variable v_srlatch_interrupt 	: std_logic_vector(4 downto 0)	:= "00000"; -- Tester  initialisere!
begin
	v_srlatch_intern 	:= interrupt_set nor v_srlatch_interrupt;
	v_srlatch_interrupt 	:= interrupt_handeled nor v_srlatch_intern;
	
	motor_interrupt		<= v_srlatch_interrupt;
end process;
-- *************************************************************************************** --



-- *************************************************************************************** --
-- *			Genererer styresingalene til tristate-bufferne
-- *************************************************************************************** --
Bus_write_enable : process (IOread, IOselect)

begin
	-------------------------------------------------------
--	if 	reset = '1' then	-- Added reset for debugging.
		-----------------------------------------------	
--		m1_write_enable <= '0';
--		m2_write_enable <= '0';
--		m3_write_enable <= '0';
--		m4_write_enable <= '0';
--		m5_write_enable <= '0';
	-------------------------------------------------------
	if 	IOread = '1' then
		-----------------------------------------------
		case 	IOselect is
			when "00001" => m1_write_enable <= '1';
			when "00010" => m2_write_enable <= '1';
			when "00100" => m3_write_enable <= '1';
			when "01000" => m4_write_enable <= '1';
			when "10000" => m5_write_enable <= '1';
			when others => null;
		-----------------------------------------------
		end case;
	-------------------------------------------------------
	elsif	IOread = '0' then
		-----------------------------------------------
		m1_write_enable <= '0';
		m2_write_enable <= '0';
		m3_write_enable <= '0';
		m4_write_enable <= '0';
		m5_write_enable <= '0';
	-------------------------------------------------------
	end if;
end process;
-- *************************************************************************************** --
-- *			Tristatebuffere for styre skriving til bussen:
-- *************************************************************************************** --
data_bus <= m1_data_out when (m1_write_enable = '1') and (reset /= '1') else (others => 'Z');
data_bus <= m2_data_out when (m2_write_enable = '1') and (reset /= '1') else (others => 'Z');
data_bus <= m3_data_out when (m3_write_enable = '1') and (reset /= '1') else (others => 'Z');
data_bus <= m4_data_out when (m4_write_enable = '1') and (reset /= '1') else (others => 'Z');
data_bus <= m5_data_out when (m5_write_enable = '1') and (reset /= '1') else (others => 'Z');
data_bus <= (others => 'Z') when m1_write_enable /= '1' and m2_write_enable /= '1' and m3_write_enable /= '1' and m4_write_enable /= '1' and m5_write_enable /= '1'; -- En test.
-- *************************************************************************************** --



-- *************************************************************************************** --
-- *	Ved endring p IOselect initieres utveksling av data.
-- *************************************************************************************** --
IOselect_poller : process (IOselect, IOwrite, IOread) is
											
	variable 	throttle  : std_logic_vector(7 downto 0) := (others => '0');
	variable 	direction : std_logic := '0';
	
	variable 	low_byte_sent	: std_logic_vector(4 downto 0) := "00000";		-- Et eksperiment:
	
	variable 	m1_speed_int, 
			m2_speed_int, 
			m3_speed_int, 
			m4_speed_int, 
			m5_speed_int 	: std_logic_vector(speed_width-1 downto 0) := (others => '0');
begin
----------------------------------------------------------------------------------------------------
		
	-----------------------------------------------------
--	if reset = '1' then			
--		throttle := (others => '0');
--		direction := '0';
--		cancel_interrupt <= (others => '0');
--		low_byte_sent := (others => '0'); 			
		
--		m1_speed_int := (others => '0');
--		m2_speed_int := (others => '0');
--		m3_speed_int := (others => '0');
--		m4_speed_int := (others => '0');
--		m5_speed_int := (others => '0');

--		m1_direction	<= '0';
--		m2_direction	<= '0';
--		m3_direction	<= '0';
--		m4_direction	<= '0';
--		m5_direction	<= '0';
		
--		m1_throttle	<= (others => '0');
--		m2_throttle	<= (others => '0');
--		m3_throttle	<= (others => '0');
--		m4_throttle	<= (others => '0');
--		m5_throttle	<= (others => '0');	
	
	-----------------------------------------------------
	-- AVR indikerer at den nsker  sende data:
	-----------------------------------------------------
	if IOwrite = '1' then-- and reset /= '1' then
		---------------------------------------------
		direction 		:= data_bus(7);
		throttle(7 downto 1) 	:= data_bus(6 downto 0);
		throttle(0) := '0';
		---------------------------------------------
		case IOselect is
			when "00001" =>
				m1_direction	<= direction;
				m1_throttle	<= throttle;
			when "00010" =>
				m2_direction	<= direction;
				m2_throttle	<= throttle;				
			when "00100" =>
				m3_direction	<= direction;
				m3_throttle	<= throttle;
			when "01000" =>
				m4_direction	<= direction;
				m4_throttle	<= throttle;
			when "10000" =>				
				m5_direction	<= direction;
				m5_throttle	<= throttle;
			when others => null;
		end case;
		---------------------------------------------
	
	-----------------------------------------------------
	-- AVR indikerer at den nsker  motta data:
	-- AVR m kalle to ganger, for  f bde low-byte og high-byte.
	-----------------------------------------------------
	elsif IOread = '1' then	--and reset /= '1' then
		---------------------------------------------
		case IOselect is
			--------------------------------------------
			when "00001" =>
				------------------------------------
				if	low_byte_sent(m1) = '0' then		-- Sjekker om low_byte er sendt for gjeldende motor.
					
					m1_speed_int := m1_current_speed;	-- Lagrer hastigheten i lokal variabel.
					m1_data_out <= m1_speed_int(7 downto 0);-- Skriver low_byte til data-bussen.
					
					low_byte_sent(m1) := '1';		-- Setter flag som indikerer at low_byte er sendt.
					m1_low_byte_sent <= '1';		-- Debug-signal
					interrupt_handeled(m1) <= '1';
				------------------------------------
				elsif	low_byte_sent(m1) = '1' then		-- Klart for  sende high_byte.

					m1_data_out <= (others => '0');		-- Nuller de verste ubrukte bitene.
					
					m1_data_out(speed_width-9 downto 0) 
					<= m1_speed_int(speed_width-1 downto 8);-- Sender resten av high_byte.
					
					low_byte_sent(m1) := '0';		-- Deaktiverer flagget igjen.
					m1_low_byte_sent <= '0';		-- Debug-signal
					interrupt_handeled(m1) <= '0';
				------------------------------------
				end if;
				
			--------------------------------------------
			when "00010" =>
				------------------------------------
				if	low_byte_sent(m2) = '0' then		-- Sjekker om low_byte er sendt for gjeldende motor.
					
					m2_speed_int := m2_current_speed;	-- Lagrer hastigheten i lokal variabel.
					m2_data_out <= m2_speed_int(7 downto 0);-- Skriver low_byte til data-bussen.
					
					low_byte_sent(m2) := '1';		-- Setter flag som indikerer at low_byte er sendt.
					m2_low_byte_sent <= '1';		-- Debug-signal
					interrupt_handeled(m2) <= '1';
				------------------------------------
				elsif	low_byte_sent(m2) = '1' then		-- Klart for  sende high_byte.

					m2_data_out <= (others => '0');		-- Nuller de verste ubrukte bitene.
					
					m2_data_out(speed_width-9 downto 0) 
					<= m2_speed_int(speed_width-1 downto 8);-- Sender resten av high_byte.
					
					low_byte_sent(m2) := '0';		-- Deaktiverer flagget igjen.
					m2_low_byte_sent <= '0';		-- Debug-signal
					interrupt_handeled(m2) <= '0';
				------------------------------------
				end if;
				
			--------------------------------------------
			when "00100" =>
				------------------------------------
				if	low_byte_sent(m3) = '0' then		-- Sjekker om low_byte er sendt for gjeldende motor.
					
					m3_speed_int := m3_current_speed;	-- Lagrer hastigheten i lokal variabel.
					m3_data_out <= m3_speed_int(7 downto 0);-- Skriver low_byte til data-bussen.
					
					low_byte_sent(m3) := '1';		-- Setter flag som indikerer at low_byte er sendt.
					m3_low_byte_sent <= '1';		-- Debug-signal
					interrupt_handeled(m3) <= '1';
				------------------------------------
				elsif	low_byte_sent(m3) = '1' then		-- Klart for  sende high_byte.

					m3_data_out <= (others => '0');		-- Nuller de verste ubrukte bitene.
					
					m3_data_out(speed_width-9 downto 0) 
					<= m3_speed_int(speed_width-1 downto 8);-- Sender resten av high_byte.
					
					low_byte_sent(m3) := '0';		-- Deaktiverer flagget igjen.
					m3_low_byte_sent <= '0';		-- Debug-signal
					interrupt_handeled(m3) <= '0';
				------------------------------------
				end if;
				
			--------------------------------------------
			when "01000" =>
				------------------------------------
				if	low_byte_sent(m4) = '0' then		-- Sjekker om low_byte er sendt for gjeldende motor.
					
					m4_speed_int := m4_current_speed;	-- Lagrer hastigheten i lokal variabel.
					m4_data_out <= m4_speed_int(7 downto 0);-- Skriver low_byte til data-bussen.
					
					low_byte_sent(m4) := '1';		-- Setter flag som indikerer at low_byte er sendt.
					m4_low_byte_sent <= '1';		-- Debug-signal
					interrupt_handeled(m4) <= '1';
				------------------------------------
				elsif	low_byte_sent(m4) = '1' then		-- Klart for  sende high_byte.

					m4_data_out <= (others => '0');		-- Nuller de verste ubrukte bitene.
					
					m4_data_out(speed_width-9 downto 0) 
					<= m4_speed_int(speed_width-1 downto 8);-- Sender resten av high_byte.
					
					low_byte_sent(m4) := '0';		-- Deaktiverer flagget igjen.
					m4_low_byte_sent <= '0';		-- Debug-signal
					interrupt_handeled(m4) <= '0';
				------------------------------------
				end if;
				
			--------------------------------------------
			when "10000" =>
				------------------------------------
				if	low_byte_sent(m5) = '0' then		-- Sjekker om low_byte er sendt for gjeldende motor.
					
					m5_speed_int := m5_current_speed;	-- Lagrer hastigheten i lokal variabel.
					m5_data_out <= m5_speed_int(7 downto 0);-- Skriver low_byte til data-bussen.
					
					low_byte_sent(m5) := '1';		-- Setter flag som indikerer at low_byte er sendt.
					m5_low_byte_sent <= '1';		-- Debug-signal
					interrupt_handeled(m5) <= '1';
				------------------------------------
				elsif	low_byte_sent(m5) = '1' then		-- Klart for  sende high_byte.

					m5_data_out <= (others => '0');		-- Nuller de verste ubrukte bitene.
					
					m5_data_out(speed_width-9 downto 0) 
					<= m5_speed_int(speed_width-1 downto 8);-- Sender resten av high_byte.
					
					low_byte_sent(m5) := '0';		-- Deaktiverer flagget igjen.
					m5_low_byte_sent <= '0';		-- Debug-signal
					interrupt_handeled(m5) <= '0';
				------------------------------------
				end if;
				
			--------------------------------------------
			when others => null;--exit;
		end case;
		---------------------------------------------

	-----------------------------------------------------
	end if;	

end process;
-- *************************************************************************************** --

m1_current_speed_int <= m1_current_speed;
m2_current_speed_int <= m2_current_speed;
m3_current_speed_int <= m3_current_speed;
m4_current_speed_int <= m4_current_speed;
m5_current_speed_int <= m5_current_speed;

interrupt_out <= motor_interrupt;	-- Sender ut interrupt.

end architecture motor_communicatior;
-- ****************************************************************************************************************** --