library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

-- *********************************************************************************************************** --
-- *													     * --
-- *	Modulen inneholder enkle kommuteringsvektorer. Oppslag baseres p hall-vektorene		     * --
-- *													     * --
-- *	Modulen br ikke implementere avansert kommutering. Det br heller skje i egne moduler.		     * --
-- *													     * --
-- *********************************************************************************************************** --


-- **************************************************************************************************************** --
entity commutator_interface is
		
	Port(	
		hall_position	: in	std_logic_vector(2 downto 0);	-- Graykodet soneangivning for rotor.
    		rotation_dir 	: in	std_logic;			-- 1 for hyre, 0 for venstre.
		commutation_out	: out	std_logic_vector(5 downto 0)	-- Kobles til drivtrinnet (h-brua).
	);

end entity commutator_interface;
-- **************************************************************************************************************** --


-- **************************************************************************************************************** --
architecture commutator of commutator_interface is

	signal hall_position_int: std_logic_vector(2 downto 0);
	signal rotation_dir_int	: std_logic;-- := '1'; -- Settes statisk inntil videre. Kobler fra porten.
	signal commutation_int	: std_logic_vector(5 downto 0);

begin

-- *********************************************** --
-- Enkeltstende prosesser for signal-oppdateringer:
-- *********************************************** --
hall_position_int<= hall_position;	-- Oppdaterer det interne signalet
commutation_out  <= commutation_int;	-- Porter tabel-oppslaget til utgangen.
rotation_dir_int <= rotation_dir;
-- *********************************************** --


-- *************************************************************************** --
-- Utfrer tabell-oppslag, og porter verdiene til utgangen for styring av H-bru.
-- Helt direkte kobling her. Tar ingen hensyn til krypstrm i H-bru.
-- Br kanskje endres fr testing p fysisk drivkrets, s ikke H-brua brenner.
-- *************************************************************************** --
commutator : commutation_int <=

	-- For rotasjon mot hyre:
	"100001" when hall_position_int = "001" and rotation_dir_int = '1' else
	"001010" when hall_position_int = "010" and rotation_dir_int = '1' else
	"100010" when hall_position_int = "011" and rotation_dir_int = '1' else
	"010100" when hall_position_int = "100" and rotation_dir_int = '1' else
	"010001" when hall_position_int = "101" and rotation_dir_int = '1' else
	"001100" when hall_position_int = "110" and rotation_dir_int = '1' else
	--"000000";

	-- For rotasjon mot venstre: (Tabell-verdier hentet fra Skjelten)
	"001100" when hall_position_int = "001" and rotation_dir_int = '0' else
	"010001" when hall_position_int = "010" and rotation_dir_int = '0' else
	"010100" when hall_position_int = "011" and rotation_dir_int = '0' else
	"100010" when hall_position_int = "100" and rotation_dir_int = '0' else
	"001010" when hall_position_int = "101" and rotation_dir_int = '0' else
	"100001" when hall_position_int = "110" and rotation_dir_int = '0' else
	"000000";
-- *********************************************** --


end architecture commutator;
-- **************************************************************************************************************** --