library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

library work;
use work.definitions.all;

-- *********************************************************************************************************** --
-- *
-- *	PWM-signalet multiplekses til korrekt utgang, basert p kommuteringsvektorene.
-- *	En forsinkelse mellom kommuteringsskift innfres, for  sikre at kortslutning
-- *	i drivtrinnet ikke forekommer.
-- *
-- *********************************************************************************************************** --


-- **************************************************************************************************************** --
entity commutation_mux_interface is
	Port(
		clk		: in std_logic;
		reset		: in std_logic;
		commutation_in	: in std_logic_vector(5 downto 0); -- Mitt select-signal.
		pwm_in		: in std_logic;
		commutation_out	: out std_logic_vector(5 downto 0)
	);
end entity commutation_mux_interface;
-- **************************************************************************************************************** --


-- **************************************************************************************************************** --
-- Tar inn den kommuterte vektoren fra commutator, og multiplekser PWM'en med disse.
-- Srger for at kun en transistor skifter om gangen, for  unng krypstrm.
-- **************************************************************************************************************** --
architecture commutation_mux of commutation_mux_interface is

	signal apply_pwm_enable	: std_logic := '0';
	signal commutation_int	: std_logic_vector(5 downto 0); -- En test
	signal pwm_int		: std_logic := '0';

	-- /\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\
	--constant off_on_delay	: integer := 1000;	-- Definerer kommuteringsforsinkelsen. Viktig!!!		-- Flyttet til packageges.vhd
	-- \/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

begin


pwm_int <= pwm_in;


-- ***************************************************** --
-- Avbryter forrige kommutering, og aktiverer neste
-- kommutering etter en definert forsinkelse.
-- ***************************************************** --

delay_pwm : process (clk, reset)	is
	
	variable delay_counter 		: integer range 0 to off_on_delay := 0;
	variable last_commutation_in 	: std_logic_vector(5 downto 0) := (others => '0');
	variable count_flag 		: std_logic := '0';	-- Telleflagg
	
begin
------------------------------------------------------
if reset = '1' then 					-- Plain resetting av signaler og variable
	----------------------------------------------
	apply_pwm_enable <= '0';			-- Deaktiverer gjeldende kommutering
	delay_counter := 0;				-- Nullstiller tellerne
	count_flag := '0';				-- Telleflagg
	last_commutation_in := "000000";		-- Tror jeg m ha med denne her for  unng at motoren har mulighet for  aldri starte etter reset.
------------------------------------------------------
elsif rising_edge(clk) then
	----------------------------------------------
	if commutation_in /= last_commutation_in then	-- Sjekker om ny kommutering foreligger fra Commutator.
		--------------------------------------
		apply_pwm_enable <= '0';		-- Deaktiverer forrige kommutering, siden ny kommutering straks skal utfres.
		last_commutation_in := commutation_in;	-- Husker verdien til neste eksekvering.
		delay_counter := 0;				-- Nuller ut tellerne fr ny forsinkelse startes opp.
		count_flag := '1';			-- Flagg settes hyt for  indikere at telling pgr.
	----------------------------------------------
	elsif count_flag = '1' then			-- Hvis ny kommutering ikke foreligger, sjekkes det om telling pgr.
		--------------------------------------
		if delay_counter = off_on_delay then	-- Sjekker om teller nsket forsinkelse er ndd.
			------------------------------
			delay_counter := 0;
			count_flag := '0';
			apply_pwm_enable <= '1';
		--------------------------------------
		else delay_counter := delay_counter + 1;
		--------------------------------------
		end if;	
	----------------------------------------------
	end if;	
------------------------------------------------------
end if;

end process;
-- ***************************************************** --





-- ******************************************************************** --
-- Aktiveres av "delay_pwm" via signalet "apply_pwm_enable".
-- Ptrykker h-brua de nye kommuteringsdata.
-- ******************************************************************** --

apply_pwm : process (apply_pwm_enable, commutation_in, pwm_int, clk) is
	
begin
	
--------------------------------------------------------------------------
if 	rising_edge(clk) then
	------------------------------------------------------------------
	if 	apply_pwm_enable = '1' then
		----------------------------------------------------------
		commutation_int(2 downto 0) <= commutation_in(2 downto 0);	-- Tilordner rett vektor til pull-down
		----------------------------------------------------------
		case commutation_in(5 downto 3) is				-- Tilordner pwm-signalet til rett utgang
			--------------------------------------------------
			when "001" => commutation_int(3) <= pwm_int; 		-- Funker, s lenge pwm_int er i sens-lista.
			when "010" => commutation_int(4) <= pwm_int;
			when "100" => commutation_int(5) <= pwm_int;
			when others => null;
			--------------------------------------------------
		end case;
	------------------------------------------------------------------
	elsif 	apply_pwm_enable = '0' then
		----------------------------------------------------------
		commutation_int <= (others => '0');
	------------------------------------------------------------------
	end if;
--------------------------------------------------------------------------
end if;

end process;
-- ******************************************************************** --


-- ********************************************************* --
-- Prosess som ptrykker ny kommuteringsvektor utgangen.
-- ********************************************************* --
commutation_out <= commutation_int;
-- ********************************************************* --


end architecture;-- commutation_mux;
-- **************************************************************************************************************** --