library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

library work;
use work.definitions.all;

-- ****************************************************************************************************************** --
entity top_entity_interface is 

port(	sys_clk		: in 	std_logic;
	rpm_clk		: in 	std_logic;
	pwm_clk		: in	std_logic;	-- Added extra
	reset		: in 	std_logic;
	
	Interrupt_out	: out 	std_logic_vector(4 downto 0);
	IOselect	: in 	std_logic_vector(4 downto 0);
	IOread		: in 	std_logic;
	IOwrite		: in 	std_logic;
	Data_bus	: inout std_logic_vector(7 downto 0);
	
	
	m1_hall_sensors	: in 	std_logic_vector(2 downto 0);
	m2_hall_sensors	: in 	std_logic_vector(2 downto 0);
	m3_hall_sensors	: in 	std_logic_vector(2 downto 0);
	m4_hall_sensors	: in 	std_logic_vector(2 downto 0);
	m5_hall_sensors	: in 	std_logic_vector(2 downto 0);
	
	m1_control	: out	std_logic_vector(5 downto 0);
	m2_control	: out	std_logic_vector(5 downto 0);
	m3_control	: out	std_logic_vector(5 downto 0);
	m4_control	: out	std_logic_vector(5 downto 0);
	m5_control	: out	std_logic_vector(5 downto 0)
);
end entity;
-- ****************************************************************************************************************** --


architecture top_entity of top_entity_interface is
	
	-- Signaler og variable

	-- ***** Components ***** --
----------------------------------------------------------------------------------------------------------------
component motor_controller_interface 
Port(	
	sys_clk		: in std_logic;		-- Systemklokke. Kjrer i frste omgang hyeste tilgjengelige.
	rpm_clk		: in std_logic;		-- 32,768 kHz for beregning av RPM i sanntid.
	pwm_clk		: in std_logic;		-- Forelpig ikke gjort noe beregninger her. Men denne bare trekker ekstra strm ved  vre for hy tror jeg.
	reset		: in std_logic;		-- Global reset av alle registre.
	
	top_hall_in	: in std_logic_vector(2 downto 0); 	-- Tilkobles hall-sensorene.
	
	top_duty_cycle	: in std_logic_vector(7 downto 0); 	-- Pdrag fra regulator (nsket dutycycle)	--throttle	
	top_rotation_dir: in std_logic;				-- nsket rotasjonsretning.	-- rotation_dir
	--top_com_angle	: in std_logic_vector(7 downto 0);	-- nsket forskyvning av kommuteringsvinekelen.

	top_div_req	: out std_logic;
	top_div_ack	: in  std_logic;
	top_dividend	: out std_logic_vector((data_width*2)-1 downto 0);
	top_divisor	: out std_logic_vector((data_width*1)-1 downto 0);
	top_result_in	: in std_logic_vector((speed_width -1) downto 0);	-- Litt usikker p bredden her. Tror dette er rett!
	
	top_speed_out	: out std_logic_vector(speed_width-1 downto 0);
	top_motor_ctrl	: out std_logic_vector(5 downto 0) 	-- Styrer transistorene i H-brua.
);
end component;
----------------------------------------------------------------------------------------------------------------
	
----------------------------------------------------------------------------------------------------------------
component motor_communication_interface
port(	-- ***** System interface ***** --
	clk		: in std_logic;
	reset		: in std_logic;
	
	-- ***** Calling interface ***** --
	IOselect	: in 	std_logic_vector(4 downto 0);	-- Interrupt som iverksetter utveksling av data. Throttle inn og Speed ut.
	IOwrite		: in 	std_logic;			-- Gir FPGA signal om at AVR er klar for mottak av data.
	IOread		: in 	std_logic;			-- Gir FPGA signal om at data blir sendt fra AVR
	Data_bus	: inout std_logic_vector(7 downto 0); 	-- Felles data-buss for sending og mottak av data.
	Interrupt_out	: out	std_logic_vector(4 downto 0);	-- Velger forelpig  ha dedikerte interrupt.
		
	-- ***** Motor interface ***** --
	m1_direction	: out	std_logic;
	m1_throttle	: out	std_logic_vector(7 downto 0);
	m1_current_speed: in	std_logic_vector(speed_width-1 downto 0);
	
	m2_direction	: out	std_logic;
	m2_throttle	: out	std_logic_vector(7 downto 0);
	m2_current_speed: in	std_logic_vector(speed_width-1 downto 0);
	
	m3_direction	: out	std_logic;
	m3_throttle	: out	std_logic_vector(7 downto 0);
	m3_current_speed: in	std_logic_vector(speed_width-1 downto 0);
		
	m4_direction	: out	std_logic;
	m4_throttle	: out	std_logic_vector(7 downto 0);
	m4_current_speed: in	std_logic_vector(speed_width-1 downto 0);
		
	m5_direction	: out	std_logic;
	m5_throttle	: out	std_logic_vector(7 downto 0);
	m5_current_speed: in	std_logic_vector(speed_width-1 downto 0)
);
end component;
----------------------------------------------------------------------------------------------------------------
	
----------------------------------------------------------------------------------------------------------------
component arbiter_interface
port (	Clock, Reset	: in std_logic;
		
	Dividend_1,
	Dividend_2,
	Dividend_3,
	Dividend_4,
	Dividend_5	: in std_logic_vector(data_width*2 - 1 downto 0);
		
	Divisor_1,
	Divisor_2,
	Divisor_3,
	Divisor_4,
	Divisor_5	: in std_logic_vector(data_width -1 downto 0);
		
	Req_1,
	Req_2,
	Req_3,
	Req_4,
	Req_5		: in std_logic;
		
	Ack_1,
	Ack_2,
	Ack_3,
	Ack_4,
	Ack_5		: out std_logic;
		
	Result_1,
	Result_2,
	Result_3,
	Result_4,
	Result_5	: out std_logic_vector(speed_width -1 downto 0);
		
	Dividend_out	: out std_logic_vector(data_width*2 - 1 downto 0);
	Divisor_out	: out std_logic_vector(data_width   - 1 downto 0);
	Result_in	: in  std_logic_vector(data_width*2 downto 0)
		
);
end component;
----------------------------------------------------------------------------------------------------------------
	
----------------------------------------------------------------------------------------------------------------
component DIV_UNIT
port (
	CLK		: in  STD_LOGIC;
	DIVIDEND	: in  STD_LOGIC_VECTOR (data_width*2 - 1 downto 0);
	DIVISOR		: in  STD_LOGIC_VECTOR (data_width   - 1 downto 0);
	DIV_RESULT	: out STD_LOGIC_VECTOR (data_width*2 downto 0)
);
end component;
----------------------------------------------------------------------------------------------------------------

	signal w_sys_clk 	: std_logic;
	signal w_rpm_clk 	: std_logic;
	signal w_pwm_clk 	: std_logic;
	signal w_reset		: std_logic;
	
--	signal w_IOselect 	: std_logic_vector(4 downto 0);
--	signal w_IOread		: std_logic;
--	signal w_IOwrite	: std_logic;
--	signal w_Data_bus	: std_logic_vector(7 downto 0);
--	signal w_Interrupt	: std_logic_vector(4 downto 0);
	
	signal w_m1_hall	: std_logic_vector(2 downto 0); 
	signal w_m1_throttle 	: std_logic_vector(7 downto 0);
	signal w_m1_direction	: std_logic;
	--signal w_m1_angle	: std_logic_vector(7 downto 0);
	signal w_m1_speed	: std_logic_vector((speed_width-1) downto 0);--	:= (others => '0');
	
	signal w_m1_req		: std_logic;-- := '0';				-- Added for init
	signal w_m1_ack		: std_logic;-- := '0';				-- Added for init
	signal w_m1_dividend	: std_logic_vector((data_width*2 -1) downto 0);-- 	:= (others => '0');
	signal w_m1_divisor	: std_logic_vector((data_width*1 -1) downto 0);-- 	:= (others => '0');
	signal w_m1_result	: std_logic_vector(speed_width -1 downto 0);-- 	:= (others => '0');
	
	signal w_m2_hall	: std_logic_vector(2 downto 0); 
	signal w_m2_throttle 	: std_logic_vector(7 downto 0);
	signal w_m2_direction	: std_logic;
	--signal w_m2_angle	: std_logic_vector(7 downto 0);
	signal w_m2_speed	: std_logic_vector((speed_width-1) downto 0);

	signal w_m2_req		: std_logic;
	signal w_m2_ack		: std_logic;
	signal w_m2_dividend	: std_logic_vector((data_width*2 -1) downto 0);
	signal w_m2_divisor	: std_logic_vector((data_width*1 -1) downto 0);
	signal w_m2_result	: std_logic_vector(speed_width -1 downto 0);
	
	signal w_m3_hall	: std_logic_vector(2 downto 0); 
	signal w_m3_throttle 	: std_logic_vector(7 downto 0);
	signal w_m3_direction	: std_logic;
	--signal w_m3_angle	: std_logic_vector(7 downto 0);
	signal w_m3_speed	: std_logic_vector((speed_width-1) downto 0);
	
	signal w_m3_req		: std_logic;
	signal w_m3_ack		: std_logic;
	signal w_m3_dividend	: std_logic_vector((data_width*2 -1) downto 0);
	signal w_m3_divisor	: std_logic_vector((data_width*1 -1) downto 0);
	signal w_m3_result	: std_logic_vector(speed_width -1 downto 0);
	
	signal w_m4_hall	: std_logic_vector(2 downto 0); 
	signal w_m4_throttle 	: std_logic_vector(7 downto 0);
	signal w_m4_direction	: std_logic;
	--signal w_m4_angle	: std_logic_vector(7 downto 0);
	signal w_m4_speed	: std_logic_vector((speed_width-1) downto 0);
	
	signal w_m4_req		: std_logic;
	signal w_m4_ack		: std_logic;
	signal w_m4_dividend	: std_logic_vector((data_width*2 -1) downto 0);
	signal w_m4_divisor	: std_logic_vector((data_width*1 -1) downto 0);
	signal w_m4_result	: std_logic_vector(speed_width -1 downto 0);
	
	signal w_m5_hall	: std_logic_vector(2 downto 0); 
	signal w_m5_throttle 	: std_logic_vector(7 downto 0);
	signal w_m5_direction	: std_logic;
	--signal w_m5_angle	: std_logic_vector(7 downto 0);
	signal w_m5_speed	: std_logic_vector((speed_width-1) downto 0);
	
	signal w_m5_req		: std_logic;
	signal w_m5_ack		: std_logic;
	signal w_m5_dividend	: std_logic_vector((data_width*2 -1) downto 0);
	signal w_m5_divisor	: std_logic_vector((data_width*1 -1) downto 0);
	signal w_m5_result	: std_logic_vector(speed_width -1 downto 0);
	
	signal w_arb_dividend	: std_logic_vector((data_width*2 -1) downto 0);
	signal w_arb_divisor	: std_logic_vector((data_width*1 -1) downto 0);
	signal w_arbiter_result	: std_logic_vector((data_width*2) downto 0);		
	--signal w_common_result	: std_logic_vector((data_width*2) downto 0);
	
	signal w_m1_ctrl	: std_logic_vector(5 downto 0);
	signal w_m2_ctrl	: std_logic_vector(5 downto 0);
	signal w_m3_ctrl	: std_logic_vector(5 downto 0);
	signal w_m4_ctrl	: std_logic_vector(5 downto 0);
	signal w_m5_ctrl	: std_logic_vector(5 downto 0);
	
begin

-- ************************************** --
communicator : motor_communication_interface
port map(
	
	clk		=> w_sys_clk,
	reset		=> w_reset,

	-- ***** Calling interface ***** --
	IOselect	=> IOselect,		--w_IOselect, 
	IOwrite		=> IOwrite,		--w_IOwrite,
	IOread		=> IOread,		--w_IOread,
	Data_bus	=> Data_bus,		--w_Data_bus, -- Kobler portene direkte isteden. Funka!!!
	Interrupt_out	=> Interrupt_out,	--w_Interrupt,
		
	-- ***** Motor interface ***** --
	m1_direction	=> w_m1_direction,
	m1_throttle	=> w_m1_throttle,
	m1_current_speed=> w_m1_speed,

	m2_direction	=> w_m2_direction,
	m2_throttle	=> w_m2_throttle,
	m2_current_speed=> w_m2_speed,
		
	m3_direction	=> w_m3_direction,
	m3_throttle	=> w_m3_throttle,
	m3_current_speed=> w_m3_speed,
		
	m4_direction	=> w_m4_direction,
	m4_throttle	=> w_m4_throttle,
	m4_current_speed=> w_m4_speed,
		
	m5_direction	=> w_m5_direction,
	m5_throttle	=> w_m5_throttle,
	m5_current_speed=> w_m5_speed
);
-- ************************************** --
	
	
-- ************************************** --
arbiter : arbiter_interface
port map(
	Clock		=> w_sys_clk,
	Reset		=> w_reset,
	
	Dividend_1	=> w_m1_dividend,
	Dividend_2	=> w_m2_dividend,
	Dividend_3	=> w_m3_dividend,
	Dividend_4	=> w_m4_dividend,
	Dividend_5	=> w_m5_dividend,
		
	Divisor_1	=> w_m1_divisor,
	Divisor_2	=> w_m2_divisor,
	Divisor_3	=> w_m3_divisor,
	Divisor_4	=> w_m4_divisor,
	Divisor_5	=> w_m5_divisor,
		
	Req_1		=> w_m1_req,
	Req_2		=> w_m2_req,
	Req_3		=> w_m3_req,
	Req_4		=> w_m4_req,
	Req_5		=> w_m5_req,
		
	Ack_1		=> w_m1_ack,
	Ack_2		=> w_m2_ack,
	Ack_3		=> w_m3_ack,
	Ack_4		=> w_m4_ack,
	Ack_5		=> w_m5_ack,
		
	Result_1	=> w_m1_result,
	Result_2	=> w_m2_result,
	Result_3	=> w_m3_result,
	Result_4	=> w_m4_result,
	Result_5	=> w_m5_result,
		
	Dividend_out	=> w_arb_dividend,
	Divisor_out	=> w_arb_divisor,
	Result_in	=> w_arbiter_result	
	
);
-- ************************************** --
		
		
-- ************************************** --
div_module : DIV_UNIT
port map(
	CLK		=> w_sys_clk,
	DIVIDEND	=> w_arb_dividend,
	DIVISOR		=> w_arb_divisor,
	DIV_RESULT	=> w_arbiter_result	-- Forelpig direkte koblet til alle motorer.
);
-- ************************************** --


-- ************************************** --
motor_1 : motor_controller_interface
port map(
	sys_clk		=> w_sys_clk,
	rpm_clk		=> w_rpm_clk,
	pwm_clk		=> w_pwm_clk,	--w_sys_clk,
	reset		=> w_reset,
	
	top_hall_in	=> w_m1_hall,
	
	top_duty_cycle	=> w_m1_throttle,
	top_rotation_dir=> w_m1_direction,
	--top_com_angle	=> w_m1_angle,

	top_div_req	=> w_m1_req,
	top_div_ack	=> w_m1_ack,
	top_dividend	=> w_m1_dividend,
	top_divisor	=> w_m1_divisor,
	top_result_in	=> w_m1_result,
		
	top_speed_out	=> w_m1_speed,
	top_motor_ctrl	=> w_m1_ctrl
);
-- ************************************** --


-- ************************************** --
motor_2 : motor_controller_interface
port map(
	sys_clk		=> w_sys_clk,
	rpm_clk		=> w_rpm_clk,
	pwm_clk		=> w_pwm_clk,	--w_sys_clk,
	reset		=> w_reset,
		
	top_hall_in	=> w_m2_hall,
	
	top_duty_cycle	=> w_m2_throttle,
	top_rotation_dir=> w_m2_direction,
	--top_com_angle	=> w_m2_angle,

	top_div_req	=> w_m2_req,
	top_div_ack	=> w_m2_ack,
	top_dividend	=> w_m2_dividend,
	top_divisor	=> w_m2_divisor,
	top_result_in	=> w_m2_result,
		
	top_speed_out	=> w_m2_speed,
	top_motor_ctrl	=> w_m2_ctrl
);
-- ************************************** --
	
	
-- ************************************** --
motor_3 : motor_controller_interface
port map(
	sys_clk		=> w_sys_clk,
	rpm_clk		=> w_rpm_clk,
	pwm_clk		=> w_pwm_clk,	--w_sys_clk,
	reset		=> w_reset,
		
	top_hall_in	=> w_m3_hall,
	
	top_duty_cycle	=> w_m3_throttle,
	top_rotation_dir=> w_m3_direction,
	--top_com_angle	=> w_m3_angle,

	top_div_req	=> w_m3_req,
	top_div_ack	=> w_m3_ack,
	top_dividend	=> w_m3_dividend,
	top_divisor	=> w_m3_divisor,
	top_result_in	=> w_m3_result,
		
	top_speed_out	=> w_m3_speed,
	top_motor_ctrl	=> w_m3_ctrl
);
-- ************************************** --
	
	
-- ************************************** --
motor_4 : motor_controller_interface
port map(
	sys_clk		=> w_sys_clk,
	rpm_clk		=> w_rpm_clk,
	pwm_clk		=> w_pwm_clk,	--w_sys_clk,
	reset		=> w_reset,
		
	top_hall_in	=> w_m4_hall,
	
	top_duty_cycle	=> w_m4_throttle,
	top_rotation_dir=> w_m4_direction,
	--top_com_angle	=> w_m4_angle,

	top_div_req	=> w_m4_req,
	top_div_ack	=> w_m4_ack,
	top_dividend	=> w_m4_dividend,
	top_divisor	=> w_m4_divisor,
	top_result_in	=> w_m4_result,	
		
	top_speed_out	=> w_m4_speed,
	top_motor_ctrl	=> w_m4_ctrl
);
-- ************************************** --
	
	
-- ************************************** --
motor_5 : motor_controller_interface
port map(
	sys_clk		=> w_sys_clk,
	rpm_clk		=> w_rpm_clk,
	pwm_clk		=> w_pwm_clk,	--w_sys_clk,
	reset		=> w_reset,
		
	top_hall_in	=> w_m5_hall,
		
	top_duty_cycle	=> w_m5_throttle,
	top_rotation_dir=> w_m5_direction,
	--top_com_angle	=> w_m5_angle,

	top_div_req	=> w_m5_req,
	top_div_ack	=> w_m5_ack,
	top_dividend	=> w_m5_dividend,
	top_divisor	=> w_m5_divisor,
	top_result_in	=> w_m5_result,
		
	top_speed_out	=> w_m5_speed,
	top_motor_ctrl	=> w_m5_ctrl
);
-- ************************************** --
	
	

	
w_sys_clk <= sys_clk;
w_rpm_clk <= rpm_clk;
w_pwm_clk <= w_pwm_clk;
w_reset   <= reset;
	
w_m1_hall <= m1_hall_sensors;
w_m2_hall <= m2_hall_sensors;
w_m3_hall <= m3_hall_sensors;
w_m4_hall <= m4_hall_sensors;
w_m5_hall <= m5_hall_sensors;

m1_control <= w_m1_ctrl;
m2_control <= w_m2_ctrl;
m3_control <= w_m3_ctrl;
m4_control <= w_m4_ctrl;
m5_control <= w_m5_ctrl;
	
--w_data_bus <= Data_bus;
--Data_bus <= w_data_bus;

end architecture;