Library IEEE;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_arith.all ;
USE work.user_pkg.all;

-- ***************************************************************************************************** --
ENTITY pwm_fpga IS
	PORT (	clock 		: in STD_LOGIC;
		reset 		: in STD_LOGIC;
		Data_value 	: in std_logic_vector(7 downto 0);	-- Set-port for duty cycle.
		pwm		: out STD_LOGIC				-- Ferdig PWM-wave ut.
	);
END pwm_fpga;
-- ***************************************************************************************************** --


-- ***************************************************************************************************** --
ARCHITECTURE arch_pwm OF pwm_fpga IS

SIGNAL reg_out 		: std_logic_vector(7 downto 0);	-- Internt signal for duty cycle.
SIGNAL cnt_out_int	: std_logic_vector(7 downto 0); -- Aner ikke!!!
SIGNAL pwm_int		: std_logic;			-- Intern pwm-wave tror jeg?
SIGNAL rco_int		: std_logic;			-- Aner ikke!!!

BEGIN


-- ************************************************************************************** --
	-- 8-bit data register to store the data values .The data values
	-- will determine the duty cycle of PWM output
-- ************************************************************************************** --
	-- Slik forstr jeg denne: Leser inn nsket duty cycle fra "data_value-porten",
	-- og legger disse tilgjengelig p "reg_out", et internt signal, for at telleren
	-- skal kunne bruke det.
-- ************************************************************************************** --
PROCESS (clock, reg_out, reset)
BEGIN

	IF (reset ='1') THEN
		reg_out <="00000000";	-- Nuller internt register.
	ELSIF (rising_edge(clock)) THEN	-- Hver gang klokka slr:
		reg_out <= data_value;	-- Porten portes til internt signal.
	END IF;

END PROCESS;
-- ************************************************************************************** --


-- ************************************************************************************** --
	--8-bit up/down counter. Counts up or down based on the pwm_int signal
	--and generates terminal count whenever counter reaches the
	--maximum value or when it transists through zero. Terminal
	--count is uesd to automatically load the data value to generate
	--different pwm out with different duty cycle
	--INC and DEC are the two functions which are used for up and
	--down counting. they are defined in sepearate user_pakge library
-- ************************************************************************************** --
PROCESS (clock, cnt_out_int, rco_int, reg_out)
BEGIN

	IF (rco_int = '1') THEN		-- Slr til hvis cnt_out_int har ndd topp/bunn, og ved reset.
		cnt_out_int <= reg_out; -- Starter  telle p nytt.

	ELSIF rising_edge(clock) THEN
		IF (rco_int = '0' and pwm_int ='1' and cnt_out_int <"11111111") THEN
			cnt_out_int <= INC(cnt_out_int);
		ELSE
			IF (rco_int ='0' and pwm_int ='0' and cnt_out_int > "00000000") THEN
				cnt_out_int <= DEC(cnt_out_int);
			END IF;
		END IF;
	END IF;

END PROCESS;
-- ************************************************************************************** --


-- ************************************************************************************** --
PROCESS(cnt_out_int, rco_int, clock, reset)
BEGIN

	IF (reset ='1') THEN
		rco_int <='1';		-- Nuller tellerne til init-verdi/ omstart av PWM.
	ELSIF rising_edge(clock) THEN
		IF ((cnt_out_int = "11111111") or (cnt_out_int ="00000000")) THEN
			rco_int <= '1';	-- Endestopp-bryter. Nuller tellerne/omstart av PWM.
		ELSE
			rco_int <= '0'; -- Deaktiverer endestopp-bryter.
		END IF;
	END IF;

END PROCESS;
-- ************************************************************************************** --


-- ************************************************************************************** --
-- Logic to Generate the PWM ouput.
PROCESS (clock,rco_int,reset)
BEGIN

	IF (reset = '1') THEN
		pwm_int <='0';
	ELSIF rising_edge(rco_int) THEN
		pwm_int <= NOT(pwm_int);
	ELSE
		pwm_int <= pwm_int;
	END IF;

END PROCESS;

pwm <= pwm_int;

END arch_pwm;
-- ***************************************************************************************************** --