-- Filename          : tb_tmp.vhd
-- Modelname         : TB_MOTOR_COMMUNICATION_INTERFACEMOTOR_COMMUNICATION_INTERFACE
-- Title             :
-- Purpose           :
-- Author(s)         : root
-- Comment           :
-- Assumptions       :
-- Limitations       :
-- Known errors      :
-- Specification ref :
-- ------------------------------------------------------------------------
-- Modification history:
-- ------------------------------------------------------------------------
-- Version  | Author | Date       | Changes made
-- ------------------------------------------------------------------------
-- 1.0      | root | 11.06.2006 | inital version


-- Testbenk med oppdaterte rutiner fra Top_module_tb2. For  sammenligne med den opprinnelige.


library IEEE, WORK;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use WORK.definitions.all;

entity TB_MOTOR_COMMUNICATION_INTERFACE is
end TB_MOTOR_COMMUNICATION_INTERFACE;

architecture BEH of TB_MOTOR_COMMUNICATION_INTERFACE is

   component MOTOR_COMMUNICATION_INTERFACE
      port(CLK                : in std_logic ;
      	   RESET	      : in std_logic ;
           IOSELECT           : in std_logic_vector ( 4 downto 0 );
           IOWRITE            : in std_logic ;
           IOREAD             : in std_logic ;
           DATA_BUS           : inout std_logic_vector ( 7 downto 0 );
           INTERRUPT_OUT      : out std_logic_vector ( 4 downto 0 );
           M1_DIRECTION       : out std_logic ;

           M1_THROTTLE        : out std_logic_vector ( 7 downto 0 );
           M1_CURRENT_SPEED   : in std_logic_vector ( speed_width - 1 downto 0 );
           M2_DIRECTION       : out std_logic ;
           M2_THROTTLE        : out std_logic_vector ( 7 downto 0 );
           M2_CURRENT_SPEED   : in std_logic_vector ( speed_width - 1 downto 0 );
           M3_DIRECTION       : out std_logic ;
           M3_THROTTLE        : out std_logic_vector ( 7 downto 0 );
           M3_CURRENT_SPEED   : in std_logic_vector ( speed_width - 1 downto 0 );
           M4_DIRECTION       : out std_logic ;
           M4_THROTTLE        : out std_logic_vector ( 7 downto 0 );
           M4_CURRENT_SPEED   : in std_logic_vector ( speed_width - 1 downto 0 );
           M5_DIRECTION       : out std_logic ;
           M5_THROTTLE        : out std_logic_vector ( 7 downto 0 );
           M5_CURRENT_SPEED   : in std_logic_vector ( speed_width - 1 downto 0 ) );

   end component;


   constant PERIOD : time := 30 ns;

   signal W_CLK                : std_logic  := '1';
   signal W_RESET	       : std_logic  := '0';
   signal W_IOSELECT           : std_logic_vector ( 4 downto 0 );
   signal W_IOWRITE            : std_logic ;
   signal W_IOREAD             : std_logic ;
   signal W_DATA_BUS           : std_logic_vector ( 7 downto 0 );
   signal W_INTERRUPT_OUT      : std_logic_vector ( 4 downto 0 );
   signal W_M1_DIRECTION       : std_logic ;
   signal W_M1_THROTTLE        : std_logic_vector ( 7 downto 0 );
   signal W_M1_CURRENT_SPEED   : std_logic_vector ( speed_width - 1 downto 0 );
   signal W_M2_DIRECTION       : std_logic ;
   signal W_M2_THROTTLE        : std_logic_vector ( 7 downto 0 );
   signal W_M2_CURRENT_SPEED   : std_logic_vector ( speed_width - 1 downto 0 );
   signal W_M3_DIRECTION       : std_logic ;
   signal W_M3_THROTTLE        : std_logic_vector ( 7 downto 0 );
   signal W_M3_CURRENT_SPEED   : std_logic_vector ( speed_width - 1 downto 0 );
   signal W_M4_DIRECTION       : std_logic ;
   signal W_M4_THROTTLE        : std_logic_vector ( 7 downto 0 );
   signal W_M4_CURRENT_SPEED   : std_logic_vector ( speed_width - 1 downto 0 );
   signal W_M5_DIRECTION       : std_logic ;
   signal W_M5_THROTTLE        : std_logic_vector ( 7 downto 0 );
   signal W_M5_CURRENT_SPEED   : std_logic_vector ( speed_width - 1 downto 0 ) ;
   
	signal data_to_write	: std_logic_vector(7 downto 0) := (others => '0');
	signal write_address 	: std_logic_vector(4 downto 0) := (others => '0');
	signal read_address 	: std_logic_vector(4 downto 0) := (others => '0');
	signal tx_high_byte	: std_logic := '0';
	signal receive_flag	: std_logic := '0';					-- Added for testing
	signal ext_last_read_adr: std_logic_vector(4 downto 0) := (others => '0');	-- Added for testing

	signal m1_data_in	: std_logic_vector(7 downto 0) := (others => '0');
	signal m2_data_in	: std_logic_vector(7 downto 0) := (others => '0');
	signal m3_data_in	: std_logic_vector(7 downto 0) := (others => '0');
	signal m4_data_in	: std_logic_vector(7 downto 0) := (others => '0');
	signal m5_data_in	: std_logic_vector(7 downto 0) := (others => '0');
	
	signal tb_m1_speed	: std_logic_vector(15 downto 0) := (others => '0'); -- kjrer full bredde for enkelhets skyld
	signal tb_m2_speed	: std_logic_vector(15 downto 0) := (others => '0'); -- kjrer full bredde for enkelhets skyld
	signal tb_m3_speed	: std_logic_vector(15 downto 0) := (others => '0'); -- kjrer full bredde for enkelhets skyld
	signal tb_m4_speed	: std_logic_vector(15 downto 0) := (others => '0'); -- kjrer full bredde for enkelhets skyld
	signal tb_m5_speed	: std_logic_vector(15 downto 0) := (others => '0'); -- kjrer full bredde for enkelhets skyld
	
	constant m1_adr		: std_logic_vector(4 downto 0) := "00001";
	constant m2_adr		: std_logic_vector(4 downto 0) := "00010";
	constant m3_adr		: std_logic_vector(4 downto 0) := "00100";
	constant m4_adr		: std_logic_vector(4 downto 0) := "01000";
	constant m5_adr		: std_logic_vector(4 downto 0) := "10000";

begin

   DUT : MOTOR_COMMUNICATION_INTERFACE
      port map(CLK                => W_CLK,
      	       RESET		  => W_RESET,
               IOSELECT           => W_IOSELECT,
               IOWRITE            => W_IOWRITE,
               IOREAD             => W_IOREAD,
               DATA_BUS           => W_DATA_BUS,
               INTERRUPT_OUT      => W_INTERRUPT_OUT,
               M1_DIRECTION       => W_M1_DIRECTION,
               M1_THROTTLE        => W_M1_THROTTLE,
               M1_CURRENT_SPEED   => W_M1_CURRENT_SPEED,
               M2_DIRECTION       => W_M2_DIRECTION,
               M2_THROTTLE        => W_M2_THROTTLE,
               M2_CURRENT_SPEED   => W_M2_CURRENT_SPEED,
               M3_DIRECTION       => W_M3_DIRECTION,
               M3_THROTTLE        => W_M3_THROTTLE,
               M3_CURRENT_SPEED   => W_M3_CURRENT_SPEED,
               M4_DIRECTION       => W_M4_DIRECTION,
               M4_THROTTLE        => W_M4_THROTTLE,
               M4_CURRENT_SPEED   => W_M4_CURRENT_SPEED,
               M5_DIRECTION       => W_M5_DIRECTION,
               M5_THROTTLE        => W_M5_THROTTLE,
               M5_CURRENT_SPEED   => W_M5_CURRENT_SPEED);

   W_CLK <= not W_CLK after PERIOD/2;

STIMULI : process
begin
	----------------------------------------------------------------------------
	W_RESET		  	<= '0';
	W_M1_CURRENT_SPEED	<= (others => '0');
	W_M2_CURRENT_SPEED	<= (others => '0');
	W_M3_CURRENT_SPEED	<= (others => '0');
	W_M4_CURRENT_SPEED	<= (others => '0');
	W_M5_CURRENT_SPEED	<= (others => '0');
	W_DATA_BUS		<= (others => 'Z'); -- Ser bedre ut med; men ikke autentisk.
	----------------------------------------------------------------------------
	wait for PERIOD;
	W_RESET	<= '1';
	wait for PERIOD;
	W_RESET	<= '0';
	wait for PERIOD;
	----------------------------------------------------------------------------
	-- Sending control-data to the motors:
	----------------------------------------------------------------------------	
	wait for period*3;
	write_address	<= "00001";
	data_to_write	<= "00000000";
	wait for period;
	write_address	<= "00000";
	wait for period;
	------------------------------
	write_address	<= "00010";
	data_to_write	<= "00000000";
	wait for period;
	write_address	<= "00000";
	wait for period;
	------------------------------
	write_address	<= "00100";
	data_to_write	<= "00000000";
	wait for period;
	write_address	<= "00000";
	wait for period;
	------------------------------
	write_address	<= "01000";
	data_to_write	<= "00000000";
	wait for period;
	write_address	<= "00000";
	wait for period;
	------------------------------	
	write_address	<= "10000";
	data_to_write	<= "00000000";
	wait for period;
	write_address	<= "00000";
	wait for period;
	------------------------------------------------------------------------------	
	wait for period*7;
	write_address	<= "00001";
	data_to_write	<= "11000001";
	wait for period;
	write_address	<= "00000";
	wait for period;
	------------------------------
	write_address	<= "00010";
	data_to_write	<= "11000010";
	wait for period;
	write_address	<= "00000";
	wait for period;
	------------------------------
	write_address	<= "00100";
	data_to_write	<= "11000100";
	wait for period;
	write_address	<= "00000";
	wait for period;
	------------------------------
	write_address	<= "01000";
	data_to_write	<= "11001000";
	wait for period;
	write_address	<= "00000";
	wait for period;
	------------------------------
	write_address	<= "10000";
	data_to_write	<= "11010000";
	wait for period;
	write_address	<= "00000";
	wait for period*4;
	----------------------------------------------------------------------------	
	-- Setting speeds to generate interrupts:
	----------------------------------------------------------------------------	
	W_M1_CURRENT_SPEED	<= "1111100000001";
	wait for period*2;
	W_M2_CURRENT_SPEED	<= "1111100000010";
	wait for period*2;
	W_M3_CURRENT_SPEED	<= "1111100000100";
	wait for period*2;
	W_M4_CURRENT_SPEED	<= "1111100001000";
	wait for period*2;
	W_M5_CURRENT_SPEED	<= "1111100010000";
	wait for period*20;
	----------------------------------------------------------------------------
	W_M1_CURRENT_SPEED	<= "0000100000001";
	W_M2_CURRENT_SPEED	<= "0000100000010";
	W_M3_CURRENT_SPEED	<= "0000100000100";
	W_M4_CURRENT_SPEED	<= "0000100001000";
	W_M5_CURRENT_SPEED	<= "0000100010000";
	----------------------------------------------------------------------------
      
	wait;
end process STIMULI;
   
   
testing : process(w_data_bus) is
begin
	if receive_flag = '1' then
		case 	ext_last_read_adr is
			-------------------------------------
			when "00001" =>	m1_data_in <= w_Data_bus;
			when "00010" =>	m2_data_in <= w_Data_bus;
			when "00100" =>	m3_data_in <= w_Data_bus;
			when "01000" =>	m4_data_in <= w_Data_bus;
			when "10000" =>	m5_data_in <= w_Data_bus;
			when others => null;
			-------------------------------------
		end case;
	end if;	
end process;
   
   
-- ******************************************************************************************* --
-- ******************************************************************************************* --
--tb_communicator : process(write_address, read_address, tx_high_byte, w_reset, receive_flag) is -- w_sys_clk
tb_communicator : process(w_clk, w_reset, write_address, read_address, tx_high_byte, receive_flag) is	--, tx_high_byte, w_reset, w_clk) is -- w_sys_clk
   
	variable last_write_address	: std_logic_vector(4 downto 0) := (others => '0');
	variable last_read_address 	: std_logic_vector(4 downto 0) := (others => '0');
	variable transmit_flag		: std_logic := '0';
--	variable receive_flag		: std_logic := '0';

begin
	-----------------------------------------------------------------------------------------------
	if	rising_edge(w_clk) then		-- Denne prosessen m ikke vre klokket. -- Lagt til klokke igjen for  teste.
		---------------------------------------------------------------------------------------
		if	w_reset = '1' then
			-------------------------------------------------------------------------------
			w_IOselect 	<= "00000";
			w_IOwrite	<= '0';
			w_IOread	<= '0';
			w_Data_bus	<= "ZZZZZZZZ";
			last_write_address	:= "00000";	-- Added recently
			last_read_address	:= "00000";	-- Added recently
			transmit_flag		:= '0';		-- Added recently
--			receive_flag		:= '0';		-- Added recently
			receive_flag		<= '0';
		---------------------------------------------------------------------------------------
		elsif	transmit_flag = '1' then	-- For  slette IOselect etter en klokkeperiode
		
			w_IOselect	<= "00000"; -- skrive ZZ?
			w_IOwrite	<= '0';
			w_Data_bus	<= "ZZZZZZZZ";
			
			transmit_flag := '0';
		---------------------------------------------------------------------------------------
		elsif	receive_flag = '1' then -- overfldig!!	 and w_IOread = '1' then	-- Lagt til w_IOread eksperimentelt.
			-------------------------------------------------------------------------------
--			case 	last_read_address is
				-------------------------------------
--				when "00001" =>	m1_data_in <= w_Data_bus;
--				when "00010" =>	m2_data_in <= w_Data_bus;
--				when "00100" =>	m3_data_in <= w_Data_bus;
--				when "01000" =>	m4_data_in <= w_Data_bus;
--				when "10000" =>	m5_data_in <= w_Data_bus;
--				when others => null;
				-------------------------------------
--			end case;
			-------------------------------------------------------------------------------
			w_IOselect	<= "00000";
			w_IOread	<= '0';

--			receive_flag	:= '0';
			receive_flag	<= '0';
		---------------------------------------------------------------------------------------
		elsif	(write_address /= last_write_address) and (write_address /= "00000") then
			
			w_IOselect	<= write_address;
			w_IOwrite	<= '1';
			w_Data_bus	<= data_to_write;

			last_write_address := write_address;

			transmit_flag := '1';			-- For  slette IOselect etter en klokkeperiode.
		---------------------------------------------------------------------------------------
		-- Trenger antakelig tx_high_byte ogs, fordi last_adr ellers vil vre lik som low_byte sin adr.
		---------------------------------------------------------------------------------------
		elsif	((read_address /= last_read_address) and (read_address /= "00000")) or tx_high_byte = '1' then -- tx ndvendig?
			
			w_IOselect 	<= read_address;
			w_IOread	<= '1';

			last_read_address := read_address;
			ext_last_read_adr <= read_address;	-- Testing
			
--			receive_flag := '1';
			receive_flag	<= '1';
		---------------------------------------------------------------------------------------
--		else	-- Denne skal srge for  frigi bussen etter bruk.
--			w_IOselect	<= "00000";
--			w_IOwrite	<= '0';
--			w_IOread	<= '0';
--			w_Data_bus	<= "ZZZZZZZZ";
		---------------------------------------------------------------------------------------
		end if;
	-----------------------------------------------------------------------------------------------
	end if;
	
end process;
-- ******************************************************************************************* --



-- ******************************************************************************************* --
-- ******************************************************************************************* --
Interrupt_handeler_proc : process(W_CLK, w_reset) is	--, w_interrupt_out) is -- Vil ikke at endringer her skal tukle til:, W_INTERRUPT_OUT) is

	variable low_byte_received 	: std_logic_vector(4 downto 0) := "00000";
	variable transfer_initiated 	: std_logic_vector(4 downto 0) := "00000";
	variable transfer_packet_flag	: std_logic_vector(4 downto 0) := "00000";
begin
	-----------------------------------------------------------------------------------------
	if	w_reset = '1' then
		---------------------------------------------------------------------------------
		low_byte_received	:= "00000";
		transfer_initiated	:= "00000";
		transfer_packet_flag	:= "00000";
		tx_high_byte		<= '0';
		read_address		<= "00000";
		
		tb_m1_speed		<= (others => '0');
		tb_m2_speed		<= (others => '0');
		tb_m3_speed		<= (others => '0');
		tb_m4_speed		<= (others => '0');
		tb_m5_speed		<= (others => '0');
	-----------------------------------------------------------------------------------------
	--if w_interrupt_out /= "00000" then
	elsif rising_edge(w_clk) then					--and w_interrupt_out /= "00000" then 	-- Fr aldri high-byte med denne.
		---------------------------------------------------------------------------------
		if 	(w_interrupt_out(m1) = '1' and transfer_packet_flag = "00000") or transfer_packet_flag(m1) = '1' then	-- Lagt til tx_init eksperimentelt.
			-------------------------------------------------------------------------
			if	transfer_initiated(m1) = '0' and low_byte_received(m1) = '0' then
	
				read_address <= m1_adr;		-- Tror dette skal funke!

				transfer_initiated(m1) 	:= '1';
				transfer_packet_flag(m1):= '1';
			-------------------------------------------------------------------------	
			elsif 	transfer_initiated(m1) = '1' and low_byte_received(m1) = '0' then
	
				tb_m1_speed(7 downto 0) <= m1_data_in;
	
				read_address <= "00000";	-- Tror dette m til for  kunne lese fra samme igjen senere.
	
				transfer_initiated(m1)	:= '0';
				low_byte_received(m1)	:= '1';
			-------------------------------------------------------------------------	
			elsif	transfer_initiated(m1) = '0' and low_byte_received(m1) = '1' then
	
				read_address <= m1_adr;

				transfer_initiated(m1) := '1';
				tx_high_byte		<= '1';	-- Dette er kanskje litt tvilsomt. Men br g greit i testbenken.
			-------------------------------------------------------------------------	
			elsif 	transfer_initiated(m1) = '1' and low_byte_received(m1) = '1' then

				tb_m1_speed(15 downto 8) <= m1_data_in;

				read_address <= "00000";	-- Tror dette m til for  kunne lese fra samme igjen senere.

				transfer_initiated(m1)	:= '0';
				low_byte_received(m1)	:= '0';
				transfer_packet_flag(m1):= '0';
				tx_high_byte		<= '0';
			-------------------------------------------------------------------------
			end if;
		---------------------------------------------------------------------------------
		elsif 	(w_interrupt_out(m2) = '1' and transfer_packet_flag = "00000") or transfer_packet_flag(m2) = '1' then	-- Lagt til tx_init eksperimentelt.
			-------------------------------------------------------------------------
			if	transfer_initiated(m2) = '0' and low_byte_received(m2) = '0' then
	
				read_address <= m2_adr;		-- Tror dette skal funke!

				transfer_initiated(m2) 	:= '1';
				transfer_packet_flag(m2):= '1';
			-------------------------------------------------------------------------	
			elsif 	transfer_initiated(m2) = '1' and low_byte_received(m2) = '0' then
	
				tb_m2_speed(7 downto 0) <= m2_data_in;
	
				read_address <= "00000";	-- Tror dette m til for  kunne lese fra samme igjen senere.
	
				transfer_initiated(m2)	:= '0';
				low_byte_received(m2)	:= '1';
			-------------------------------------------------------------------------	
			elsif	transfer_initiated(m2) = '0' and low_byte_received(m2) = '1' then
	
				read_address <= m2_adr;

				transfer_initiated(m2) := '1';
				tx_high_byte		<= '1';	-- Dette er kanskje litt tvilsomt. Men br g greit i testbenken.
			-------------------------------------------------------------------------	
			elsif 	transfer_initiated(m2) = '1' and low_byte_received(m2) = '1' then

				tb_m2_speed(15 downto 8) <= m2_data_in;

				read_address <= "00000";	-- Tror dette m til for  kunne lese fra samme igjen senere.

				transfer_initiated(m2)	:= '0';
				low_byte_received(m2)	:= '0';
				transfer_packet_flag(m2):= '0';
				tx_high_byte		<= '0';
			-------------------------------------------------------------------------
			end if;
		---------------------------------------------------------------------------------
		elsif 	(w_interrupt_out(m3) = '1' and transfer_packet_flag = "00000") or transfer_packet_flag(m3) = '1' then	-- Lagt til tx_init eksperimentelt.
			-------------------------------------------------------------------------
			if	transfer_initiated(m3) = '0' and low_byte_received(m3) = '0' then
	
				read_address <= m3_adr;		-- Tror dette skal funke!

				transfer_initiated(m3) 	:= '1';
				transfer_packet_flag(m3):= '1';
			-------------------------------------------------------------------------	
			elsif 	transfer_initiated(m3) = '1' and low_byte_received(m3) = '0' then
	
				tb_m3_speed(7 downto 0) <= m3_data_in;
	
				read_address <= "00000";	-- Tror dette m til for  kunne lese fra samme igjen senere.
	
				transfer_initiated(m3)	:= '0';
				low_byte_received(m3)	:= '1';
			-------------------------------------------------------------------------	
			elsif	transfer_initiated(m3) = '0' and low_byte_received(m3) = '1' then
	
				read_address <= m3_adr;

				transfer_initiated(m3) := '1';
				tx_high_byte		<= '1';	-- Dette er kanskje litt tvilsomt. Men br g greit i testbenken.
			-------------------------------------------------------------------------	
			elsif 	transfer_initiated(m3) = '1' and low_byte_received(m3) = '1' then

				tb_m3_speed(15 downto 8) <= m3_data_in;

				read_address <= "00000";	-- Tror dette m til for  kunne lese fra samme igjen senere.

				transfer_initiated(m3)	:= '0';
				low_byte_received(m3)	:= '0';
				transfer_packet_flag(m3):= '0';
				tx_high_byte		<= '0';
			-------------------------------------------------------------------------
			end if;
		---------------------------------------------------------------------------------
		elsif 	(w_interrupt_out(m4) = '1' and transfer_packet_flag = "00000") or transfer_packet_flag(m4) = '1' then	-- Lagt til tx_init eksperimentelt.
			-------------------------------------------------------------------------
			if	transfer_initiated(m4) = '0' and low_byte_received(m4) = '0' then
	
				read_address <= m4_adr;		-- Tror dette skal funke!

				transfer_initiated(m4) 	:= '1';
				transfer_packet_flag(m4):= '1';
			-------------------------------------------------------------------------	
			elsif 	transfer_initiated(m4) = '1' and low_byte_received(m4) = '0' then
	
				tb_m4_speed(7 downto 0) <= m4_data_in;
	
				read_address <= "00000";	-- Tror dette m til for  kunne lese fra samme igjen senere.
	
				transfer_initiated(m4)	:= '0';
				low_byte_received(m4)	:= '1';
			-------------------------------------------------------------------------	
			elsif	transfer_initiated(m4) = '0' and low_byte_received(m4) = '1' then
	
				read_address <= m4_adr;

				transfer_initiated(m4) := '1';
				tx_high_byte		<= '1';	-- Dette er kanskje litt tvilsomt. Men br g greit i testbenken.
			-------------------------------------------------------------------------	
			elsif 	transfer_initiated(m4) = '1' and low_byte_received(m4) = '1' then

				tb_m4_speed(15 downto 8) <= m4_data_in;

				read_address <= "00000";	-- Tror dette m til for  kunne lese fra samme igjen senere.

				transfer_initiated(m4)	:= '0';
				low_byte_received(m4)	:= '0';
				transfer_packet_flag(m4):= '0';
				tx_high_byte		<= '0';
			-------------------------------------------------------------------------
			end if;
		---------------------------------------------------------------------------------
		elsif 	(w_interrupt_out(m5) = '1' and transfer_packet_flag = "00000") or transfer_packet_flag(m5) = '1' then	-- Lagt til tx_init eksperimentelt.
			-------------------------------------------------------------------------
			if	transfer_initiated(m5) = '0' and low_byte_received(m5) = '0' then
	
				read_address <= m5_adr;		-- Tror dette skal funke!

				transfer_initiated(m5) 	:= '1';
				transfer_packet_flag(m5):= '1';
			-------------------------------------------------------------------------	
			elsif 	transfer_initiated(m5) = '1' and low_byte_received(m5) = '0' then
	
				tb_m5_speed(7 downto 0) <= m5_data_in;
	
				read_address <= "00000";	-- Tror dette m til for  kunne lese fra samme igjen senere.
	
				transfer_initiated(m5)	:= '0';
				low_byte_received(m5)	:= '1';
			-------------------------------------------------------------------------	
			elsif	transfer_initiated(m5) = '0' and low_byte_received(m5) = '1' then
	
				read_address <= m5_adr;

				transfer_initiated(m5) := '1';
				tx_high_byte		<= '1';	-- Dette er kanskje litt tvilsomt. Men br g greit i testbenken.
			-------------------------------------------------------------------------	
			elsif 	transfer_initiated(m5) = '1' and low_byte_received(m5) = '1' then

				tb_m5_speed(15 downto 8) <= m5_data_in;

				read_address <= "00000";	-- Tror dette m til for  kunne lese fra samme igjen senere.

				transfer_initiated(m5)	:= '0';
				low_byte_received(m5)	:= '0';
				transfer_packet_flag(m5):= '0';
				tx_high_byte		<= '0';
			-------------------------------------------------------------------------
			end if;
		end if;
	-----------------------------------------------------------------------------------------
	end if;

end process;
-- ******************************************************************************************* --
   

end BEH;