-- Filename          : tb_tmp.vhd
-- Modelname         : TB_HALL_MONITOR_INTERFACE
-- Title             :
-- Purpose           :
-- Author(s)         : root
-- Comment           :
-- Assumptions       :
-- Limitations       :
-- Known errors      :
-- Specification ref :
-- ------------------------------------------------------------------------
-- Modification history:
-- ------------------------------------------------------------------------
-- Version  | Author | Date       | Changes made
-- ------------------------------------------------------------------------
-- 1.0      | root | 09.06.2006 | inital version


library IEEE, WORK;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.Numeric_STD.all;
use WORK.NUMERIC_SIGNED.ALL; -- Inneholder divisjon av std_logic_vector() Lagt til.
use WORK.hall_definitions.all;
use WORK.definitions.all;
-- Added:


entity TB_HALL_MONITOR_INTERFACE is
end TB_HALL_MONITOR_INTERFACE;

architecture BEH of TB_HALL_MONITOR_INTERFACE is

   component HALL_MONITOR_INTERFACE
      port(SYS_CLK        : in std_logic ;
           RPM_CLK        : in std_logic ;
           RESET          : in std_logic ;
           HALL_IN        : in std_logic_vector ( 2 downto 0 );
           HALL_OUT       : out std_logic_vector ( 2 downto 0 );
           SPEED          : out std_logic_vector ( speed_width - 1 downto 0 );
           DIVISION_REQ   : out std_logic ;
           DIVISION_ACK   : in std_logic ;
           DIVIDEND_OUT   : out std_logic_vector ( ( data_width*2 ) - 1 downto 0 );
           DIVISOR_OUT    : out std_logic_vector ( ( data_width*1 ) - 1 downto 0 );
           RESULT_IN      : in std_logic_vector ( (  data_width*2 ) - 1 downto 0 ); 
           rpm_counter	  : out std_logic_vector(8 downto 0) := (others => '0'));

   end component;


   constant TB_PERIOD	: time := 10 ns;
   constant PERIOD 	: time := 30 ns; 	-- Gir 33,33 MHz klokke.
   constant RT_PERIOD 	: time := 30518 ns;	--305175781 ps;--	30518 ns; 	-- Gir 32,7675 kHz klokke.
   -- ***** Motorhastigheter: ***** --
   --constant MOTOR_FQ	: time := 100 us;	-- Gir 100.000 RPM, eller 1666,67 RPS.
   constant MOTOR_FQ	: time := 1 ms;		-- Gir 10.000 RPM, eller 166,67 RPS.
   --constant MOTOR_FQ	: time := 10 ms;	-- Gir 1.000 RPM, eller 16,67 RPS.
   --constant MOTOR_FQ	: time := 416666 ns;	-- Gir 24.000 RPM. (400 RPS)
   --constant MOTOR_FQ	: time := 12500000 ns;	-- Gir 800 RPM. 

   signal W_TB_CLK	   : std_logic := '0';
   signal W_SYS_CLK        : std_logic := '0';
   signal W_RPM_CLK        : std_logic := '0';
   signal W_RESET          : std_logic := '0';
   signal W_HALL_IN        : std_logic_vector ( 2 downto 0 );
   signal W_HALL_OUT       : std_logic_vector ( 2 downto 0 );
   signal W_SPEED          : std_logic_vector ( speed_width - 1 downto 0 );
   signal W_DIVISION_REQ   : std_logic := '0';
   signal W_DIVISION_ACK   : std_logic := '0';
   signal W_DIVIDEND_OUT   : std_logic_vector ( ( data_width*2 ) - 1 downto 0 );
   signal W_DIVISOR_OUT    : std_logic_vector ( ( data_width*1 ) - 1 downto 0 );
   signal W_RESULT_IN      : std_logic_vector ( ( data_width*2 ) - 1 downto 0 ) := (others => '0');
   signal W_rpm_counter:std_logic_vector(8 downto 0) := (others => '0');
--   signal W_RPM		   : std_logic_vector(21 downto 0) := (others => '0'); -- Bare for  f RPM opp i simuleringen.

begin

   DUT : HALL_MONITOR_INTERFACE
      port map(SYS_CLK        => W_SYS_CLK,
               RPM_CLK        => W_RPM_CLK,
               RESET          => W_RESET,
               HALL_IN        => W_HALL_IN,
               HALL_OUT       => W_HALL_OUT,
               SPEED          => W_SPEED,
               DIVISION_REQ   => W_DIVISION_REQ,
               DIVISION_ACK   => W_DIVISION_ACK,
               DIVIDEND_OUT   => W_DIVIDEND_OUT,
               DIVISOR_OUT    => W_DIVISOR_OUT,
               RESULT_IN      => W_RESULT_IN,
               rpm_counter    => W_rpm_counter);

   W_TB_CLK  <= not W_TB_CLK after TB_PERIOD/2;
   
   W_SYS_CLK <= not W_SYS_CLK after PERIOD/2;
   
   W_RPM_CLK <= not W_RPM_CLK after RT_PERIOD/2;

   STIMULI : process
   begin
   --   W_SYS_CLK        <= '0';
   --   W_RPM_CLK        <= '0';
   --   W_RESET          <= '0';
      W_HALL_IN        <= (others => '0');

	W_RESET		<= '1';
	
	wait for PERIOD;
	
	W_RESET		<= '0';

      wait for PERIOD;

      	while (true) loop
      	
      		W_HALL_IN <= "100";
      		wait for MOTOR_FQ;
      		
      		W_HALL_IN <= "110";
      		wait for MOTOR_FQ;
      
      		W_HALL_IN <= "010";
      		wait for MOTOR_FQ;
      
      		W_HALL_IN <= "011";
      		wait for MOTOR_FQ;
      
      		W_HALL_IN <= "001";
      		wait for MOTOR_FQ;
      
      		W_HALL_IN <= "101";
      		wait for MOTOR_FQ;
      		


	end loop;
      
      
     -- wait;
   end process STIMULI;
   
   
   STIMULI_2 : process(W_TB_CLK)	--(W_DIVISION_REQ) --
--   STIMULI_2 : process	--(W_DIVISION_REQ) --
   
   	variable result 	: std_logic_vector((data_width*2)-1 downto 0) := (others => '0');
   	variable dividend 	: std_logic_vector((data_width*2)-1 downto 0) := (others => '0');
   	variable divisor	: std_logic_vector((data_width*1)-1 downto 0) := (others => '0');
   begin
   
   --   W_DIVISION_ACK   <= '0';
   --   W_RESULT_IN      <= (others => '0');
   
   	--while (true) loop
   		
   	if rising_edge(W_TB_CLK) then
   	
   		if W_DIVISION_REQ = '1' then
   			
   			W_DIVISION_ACK <= '1';
   		
   		elsif W_DIVISION_ACK = '1' then
   		
   			dividend := W_DIVIDEND_OUT;
   			divisor := W_DIVISOR_OUT;
   			result := "/" (dividend,divisor);
   			
   			W_RESULT_IN <= result;
   			W_DIVISION_ACK <= '0';
   			
   		else null;
   		end if;
   		
   	end if;
   	
   	--end loop;
   
   	--wait;
   end process STIMULI_2;
   
   
   --rpm_estimator : process(W_SPEED) is
   
   --	variable temp1 : std_logic_vector(speed_width-1 downto 0) := (others => '0');
   --	variable temp2 : std_logic_vector(speed_width-1 downto 0) := (others => '0');
   --begin
   
   --	temp1 := W_SPEED;
   
   --	temp2(18 downto 0) <= ("*"(temp1, "111100"));
   	
   --	W_RPM <= temp2;
   
   --end process;
   
   
   
   --temp_2(10 downto 0) := ("*"(memory, "10"));
   
   --SPEED_MULTIPLIER : process(W_SPEED) is
   
   --begin
   
   --end process;
   

end BEH;

--configuration CFG_TB_HALL_MONITOR_INTERFACE of TB_HALL_MONITOR_INTERFACE is
--   for BEH
--   end for;
--end CFG_TB_HALL_MONITOR_INTERFACE;