-- Filename          : tb_tmp.vhd
-- Modelname         : TB_MOTOR_CONTROLLER_INTERFACE
-- Title             :
-- Purpose           :
-- Author(s)         : root
-- Comment           :
-- Assumptions       :
-- Limitations       :
-- Known errors      :
-- Specification ref :
-- ------------------------------------------------------------------------
-- Modification history:
-- ------------------------------------------------------------------------
-- Version  | Author | Date       | Changes made
-- ------------------------------------------------------------------------
-- 1.0      | root | 16.06.2006 | inital version


library IEEE, WORK;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use WORK.definitions.all;
use WORK.NUMERIC_SIGNED.ALL; -- Inneholder divisjon av std_logic_vector() Lagt til.

entity TB_MOTOR_CONTROLLER_INTERFACE is
end TB_MOTOR_CONTROLLER_INTERFACE;

architecture BEH of TB_MOTOR_CONTROLLER_INTERFACE is

   component MOTOR_CONTROLLER_INTERFACE
      port(SYS_CLK            : in std_logic ;
           RPM_CLK            : in std_logic ;
           PWM_CLK            : in std_logic ;
           RESET              : in std_logic ;
           TOP_HALL_IN        : in std_logic_vector ( 2 downto 0 );
           TOP_DUTY_CYCLE     : in std_logic_vector ( 7 downto 0 );
           TOP_ROTATION_DIR   : in std_logic ;
           TOP_DIV_REQ        : out std_logic ;
           TOP_DIV_ACK        : in  std_logic ;
           TOP_DIVIDEND       : out std_logic_vector ((data_width * 2 ) - 1 downto 0 );
           TOP_DIVISOR        : out std_logic_vector ((data_width * 1 ) - 1 downto 0 );
           TOP_RESULT_IN      : in  std_logic_vector ((data_width * 2 ) - 1 downto 0 );
           TOP_SPEED_OUT      : out std_logic_vector (speed_width - 1 downto 0 );
           TOP_MOTOR_CTRL     : out std_logic_vector ( 5 downto 0 ) );

   end component;

   constant PERIOD 	: time := 30 ns; 	-- Gir 33,33 MHz klokke.
   constant RT_PERIOD 	: time := 30518 ns; 	-- Gir 32,7675 kHz klokke.
   constant PWM_PERIOD	: time := 30 ns;	-- Kjrer samme som resten forelpig.
   constant TP_PERIOD	: time := 5 ns;
   
   -- ***** Motorhastigheter: ***** --
   --constant MOTOR_FQ	: time := 100 us;	-- Gir 100.000 RPM, eller 1666,67 RPS.
   constant MOTOR_FQ	: time := 1 ms;		-- Gir 10.000 RPM, eller 166,67 RPS.
   --constant MOTOR_FQ	: time := 10 ms;	-- Gir 1.000 RPM, eller 16,67 RPS.
   --constant MOTOR_FQ	: time := 416666 ns;	-- Gir 24.000 RPM. (400 RPS)
   --constant MOTOR_FQ	: time := 12500000 ns;	-- Gir 800 RPM. 

   signal W_SYS_CLK            : std_logic := '0';
   signal W_RPM_CLK            : std_logic := '0';
   signal W_PWM_CLK            : std_logic := '0';
   signal W_TB_CLK	       : std_logic := '0';
   signal W_RESET              : std_logic ;
   signal W_TOP_HALL_IN        : std_logic_vector ( 2 downto 0 );
   signal W_TOP_DUTY_CYCLE     : std_logic_vector ( 7 downto 0 );
   signal W_TOP_ROTATION_DIR   : std_logic ;
   signal W_TOP_DIV_REQ        : std_logic ;
   signal W_TOP_DIV_ACK        : std_logic ;
   signal W_TOP_DIVIDEND       : std_logic_vector ( ( data_width * 2 ) - 1 downto 0 );
   signal W_TOP_DIVISOR        : std_logic_vector ( ( data_width * 1 ) - 1 downto 0 );
   signal W_TOP_RESULT_IN      : std_logic_vector ( ( data_width * 2 ) - 1 downto 0 );
   signal W_TOP_SPEED_OUT      : std_logic_vector ( speed_width - 1 downto 0 );
   signal W_TOP_MOTOR_CTRL     : std_logic_vector ( 5 downto 0 ) ;

begin

-- **************************************************
   DUT : motor_controller_interface		--MOTOR_CONTROLLER_INTERFACE
      port map(SYS_CLK            => W_SYS_CLK,
               RPM_CLK            => W_RPM_CLK,
               PWM_CLK            => W_PWM_CLK,
               RESET              => W_RESET,
               TOP_HALL_IN        => W_TOP_HALL_IN,
               TOP_DUTY_CYCLE     => W_TOP_DUTY_CYCLE,
               TOP_ROTATION_DIR   => W_TOP_ROTATION_DIR,
               TOP_DIV_REQ        => W_TOP_DIV_REQ,
               TOP_DIV_ACK        => W_TOP_DIV_ACK,
               TOP_DIVIDEND       => W_TOP_DIVIDEND,
               TOP_DIVISOR        => W_TOP_DIVISOR,
               TOP_RESULT_IN      => W_TOP_RESULT_IN,
               TOP_SPEED_OUT      => W_TOP_SPEED_OUT,
               TOP_MOTOR_CTRL     => W_TOP_MOTOR_CTRL);
-- **************************************************

   W_SYS_CLK <= not W_SYS_CLK after PERIOD/2;
   W_RPM_CLK <= not W_RPM_CLK after RT_PERIOD/2;
   W_PWM_CLK <= not W_PWM_CLK after PWM_PERIOD/2;
   W_TB_CLK  <= not W_TB_CLK after TP_PERIOD/2;

-- *********************************************** --
   STIMULI : process
   begin
      W_RESET              <= '0';
      W_TOP_HALL_IN        <= (others => '0');
      W_TOP_DUTY_CYCLE     <= (others => '0');
      W_TOP_ROTATION_DIR   <= '0';
--      W_TOP_DIV_ACK        <= '0';
--      W_TOP_RESULT_IN      <= (others => '0');
      wait for PERIOD;
	W_RESET <= '1';
	wait for PERIOD;
	W_RESET <= '0';
	wait for PERIOD;
	
    	W_TOP_DUTY_CYCLE <= "10000000";
      	W_TOP_ROTATION_DIR <= '1';
	wait for PERIOD;
	
      while true loop

	W_TOP_HALL_IN   <= "100"; wait for MOTOR_FQ;
	W_TOP_HALL_IN   <= "110"; wait for MOTOR_FQ;
	W_TOP_HALL_IN   <= "010"; wait for MOTOR_FQ;
	W_TOP_HALL_IN   <= "011"; wait for MOTOR_FQ;
	W_TOP_HALL_IN   <= "001"; wait for MOTOR_FQ;
	W_TOP_HALL_IN   <= "101"; wait for MOTOR_FQ;
      	
      end loop;
      
      --wait;
   end process STIMULI;
-- *********************************************** --


-- *********************************************** --   
STIMULI_2 : process(W_TB_CLK)	--(W_DIVISION_REQ) --
   
	variable result 	: std_logic_vector((data_width*2)-1 downto 0) := (others => '0');
	variable dividend 	: std_logic_vector((data_width*2)-1 downto 0) := (others => '0');
	variable divisor	: std_logic_vector((data_width*1)-1 downto 0) := (others => '0');
begin

	if rising_edge(W_TB_CLK) then

		if W_TOP_DIV_REQ = '1' then
      			
			W_TOP_DIV_ACK <= '1';

		elsif W_TOP_DIV_ACK = '1' then
      		
			dividend := W_TOP_DIVIDEND;
			divisor := W_TOP_DIVISOR;
			result := "/" (dividend,divisor);
      			
			W_TOP_RESULT_IN <= result;
			W_TOP_DIV_ACK <= '0';
      			
		else null;
		end if;
      		
      	end if;

end process STIMULI_2;
-- *********************************************** --


end BEH;


--configuration CFG_TB_MOTOR_CONTROLLER_INTERFACE of TB_MOTOR_CONTROLLER_INTERFACE is
--   for BEH
--   end for;
--end CFG_TB_MOTOR_CONTROLLER_INTERFACE;