-- Filename          : tb_tmp.vhd
-- Modelname         : TB_TOP_ENTITY_INTERFACE;
-- Title             :
-- Purpose           :
-- Author(s)         : root
-- Comment           :
-- Assumptions       :
-- Limitations       :
-- Known errors      :
-- Specification ref :
-- ------------------------------------------------------------------------
-- Modification history:
-- ------------------------------------------------------------------------
-- Version  | Author | Date       | Changes made
-- ------------------------------------------------------------------------
-- 1.0      | root | 21.06.2006 | inital version


library IEEE, WORK;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use WORK.definitions.all;

entity TB_TOP_ENTITY_INTERFACE is
end TB_TOP_ENTITY_INTERFACE;

-- ********************************************************************************************************************************************** --
architecture TB_2 of TB_TOP_ENTITY_INTERFACE is
	-----------------------------------------------------------------
	component TOP_ENTITY_INTERFACE
	port(	SYS_CLK           : in std_logic ;
		RPM_CLK           : in std_logic ;
		PWM_CLK		  : in std_logic ;
		RESET             : in std_logic ;

		INTERRUPT_OUT     : out std_logic_vector ( 4 downto 0 );
		IOSELECT          : in std_logic_vector ( 4 downto 0 );
		IOREAD            : in std_logic ;
		IOWRITE           : in std_logic ;
		DATA_BUS          : inout std_logic_vector ( 7 downto 0 );

		M1_HALL_SENSORS   : in std_logic_vector ( 2 downto 0 );
		M2_HALL_SENSORS   : in std_logic_vector ( 2 downto 0 );
		M3_HALL_SENSORS   : in std_logic_vector ( 2 downto 0 );
		M4_HALL_SENSORS   : in std_logic_vector ( 2 downto 0 );
		M5_HALL_SENSORS   : in std_logic_vector ( 2 downto 0 );

		M1_CONTROL        : out std_logic_vector ( 5 downto 0 );
		M2_CONTROL        : out std_logic_vector ( 5 downto 0 );
		M3_CONTROL        : out std_logic_vector ( 5 downto 0 );
		M4_CONTROL        : out std_logic_vector ( 5 downto 0 );
		M5_CONTROL        : out std_logic_vector ( 5 downto 0 ) );
	end component;
	-----------------------------------------------------------------
	
	constant PERIOD 	: time := 30 ns; 	-- Gir 33,33 MHz klokke.
	constant RT_PERIOD 	: time := 30518 ns; 	-- Gir 32,7675 kHz klokke.   
	
	constant m1_adr		: std_logic_vector(4 downto 0) := "00001";
	constant m2_adr		: std_logic_vector(4 downto 0) := "00010";
	constant m3_adr		: std_logic_vector(4 downto 0) := "00100";
	constant m4_adr		: std_logic_vector(4 downto 0) := "01000";
	constant m5_adr		: std_logic_vector(4 downto 0) := "10000";

	signal W_SYS_CLK	: std_logic := '1';
	signal W_RPM_CLK	: std_logic := '1';
	signal W_PWM_CLK	: std_logic := '1';	-- Added
	signal W_RESET		: std_logic ;
	signal W_INTERRUPT_OUT	: std_logic_vector ( 4 downto 0 ) := (others => '0');
	signal W_IOSELECT	: std_logic_vector ( 4 downto 0 ) := "00000";
	signal W_IOREAD		: std_logic := '0';
	signal W_IOWRITE	: std_logic := '0';
	signal W_DATA_BUS          : std_logic_vector ( 7 downto 0 );
	signal w_m1_hall_sensors: std_logic_vector ( 2 downto 0 ) := (others => '0');
	signal w_m2_hall_sensors: std_logic_vector ( 2 downto 0 ) := (others => '0');
	signal w_m3_hall_sensors: std_logic_vector ( 2 downto 0 ) := (others => '0');
	signal w_m4_hall_sensors: std_logic_vector ( 2 downto 0 ) := (others => '0');
	signal w_m5_hall_sensors: std_logic_vector ( 2 downto 0 ) := (others => '0');
	signal w_m1_control	: std_logic_vector ( 5 downto 0 ) := (others => '0');
	signal w_m2_control	: std_logic_vector ( 5 downto 0 ) := (others => '0');
	signal w_m3_control	: std_logic_vector ( 5 downto 0 ) := (others => '0');
	signal w_m4_control	: std_logic_vector ( 5 downto 0 ) := (others => '0');
	signal w_m5_control	: std_logic_vector ( 5 downto 0 ) := (others => '0');
	
	signal tb_m1_speed	: std_logic_vector(15 downto 0) := (others => '0'); -- kjrer full bredde for enkelhets skyld
	signal tb_m2_speed	: std_logic_vector(15 downto 0) := (others => '0'); -- kjrer full bredde for enkelhets skyld
	signal tb_m3_speed	: std_logic_vector(15 downto 0) := (others => '0'); -- kjrer full bredde for enkelhets skyld
	signal tb_m4_speed	: std_logic_vector(15 downto 0) := (others => '0'); -- kjrer full bredde for enkelhets skyld
	signal tb_m5_speed	: std_logic_vector(15 downto 0) := (others => '0'); -- kjrer full bredde for enkelhets skyld
	
	signal data_to_write	: std_logic_vector(7 downto 0) := (others => '0');
	signal write_address 	: std_logic_vector(4 downto 0) := (others => '0');
	signal read_address 	: std_logic_vector(4 downto 0) := (others => '0');
	signal tx_high_byte	: std_logic := '0';

	signal m1_data_in	: std_logic_vector(7 downto 0) := (others => '0');
	signal m2_data_in	: std_logic_vector(7 downto 0) := (others => '0');
	signal m3_data_in	: std_logic_vector(7 downto 0) := (others => '0');
	signal m4_data_in	: std_logic_vector(7 downto 0) := (others => '0');
	signal m5_data_in	: std_logic_vector(7 downto 0) := (others => '0');
	
	-- ************** Motorhastigheter: ************** --
	constant MOTOR_FQ_100k	: time := 100 us;	-- Gir 100.000 RPM, eller 1666,67 RPS.
	constant MOTOR_FQ_10k	: time := 1 ms;		-- Gir 10.000 RPM, eller 166,67 RPS.
	constant MOTOR_FQ_1k	: time := 10 ms;	-- Gir 1.000 RPM, eller 16,67 RPS.
	constant MOTOR_FQ_24k	: time := 416666 ns;	-- Gir 24.000 RPM. (400 RPS)
	constant MOTOR_FQ_08k	: time := 12500000 ns;	-- Gir 800 RPM. 

begin
	-----------------------------------------------
	DUT : TOP_ENTITY_INTERFACE
	port map(SYS_CLK	=> W_SYS_CLK,
		RPM_CLK		=> W_RPM_CLK,
		PWM_CLK		=> W_PWM_CLK,
		RESET		=> W_RESET,
               
		INTERRUPT_OUT	=> W_INTERRUPT_OUT,
		IOSELECT	=> W_IOSELECT,
		IOREAD		=> W_IOREAD,
		IOWRITE		=> W_IOWRITE,

		DATA_BUS	=> W_DATA_BUS,
               
		M1_HALL_SENSORS	=> w_m1_hall_sensors,
		M2_HALL_SENSORS	=> w_m2_hall_sensors,
		M3_HALL_SENSORS	=> w_m3_hall_sensors,
		M4_HALL_SENSORS	=> w_m4_hall_sensors,
		M5_HALL_SENSORS	=> w_m5_hall_sensors,
		
		M1_CONTROL	=> w_m1_control,
		M2_CONTROL	=> w_m2_control,
		M3_CONTROL	=> w_m3_control,
		M4_CONTROL	=> w_m4_control,
		M5_CONTROL	=> w_m5_control);
	-----------------------------------------------
	
	W_SYS_CLK <= not W_SYS_CLK after PERIOD/2;

	W_RPM_CLK <= not W_RPM_CLK after RT_PERIOD/2;
	
	W_PWM_CLK <= not W_PWM_CLK after PERIOD/2;	-- 33 MHz

-- ******************************************************************** --
-- ******************************************************************** --
GENERAL_STIMULI : process
begin
	----------------------------------------------------------------------------
   	W_RESET             <= '0';
   	wait for PERIOD;
   	W_RESET             <= '1';
   	wait for PERIOD*10;
   	W_RESET             <= '0';
   	wait for PERIOD*5;
	----------------------------------------------------------------------------
	-- Sending control-data to the motors:
	----------------------------------------------------------------------------	
	wait for period*7;
	write_address	<= "00001";
	data_to_write	<= "11000001";
	wait for period;
	write_address	<= "00000";
	wait for period;
	------------------------------
	write_address	<= "00010";
	data_to_write	<= "11000010";
	wait for period;
	write_address	<= "00000";
	wait for period;
	------------------------------
	write_address	<= "00100";
	data_to_write	<= "11000100";
	wait for period;
	write_address	<= "00000";
	wait for period;
	------------------------------
	write_address	<= "01000";
	data_to_write	<= "11001000";
	wait for period;
	write_address	<= "00000";
	wait for period;
	------------------------------
	write_address	<= "10000";
	data_to_write	<= "11010000";
	wait for period;
	write_address	<= "00000";
	wait for period*4;
	----------------------------------------------------------------------------	

   	
--   	wait for 2 ms;
--   	read_address <= "00001";
--   	wait for 30 ns;
--   	read_address <= "00000";
--   	wait for 90 ns;
--   	read_address <= "00001";
--   	wait for 30 ns;
--   	read_address <= "00000";
   	

--   	while true loop
--   	wait for 1000 us;
--   	end loop;
   	wait;
end process;
-- ******************************************************************** --


-- ******************************************************************************************* --
-- ******************************************************************************************* --
tb_communicator : process(write_address, read_address, tx_high_byte, w_reset) is -- w_sys_clk
   
	variable last_write_address	: std_logic_vector(4 downto 0) := (others => '0');
	variable last_read_address 	: std_logic_vector(4 downto 0) := (others => '0');
	variable transmit_flag		: std_logic := '0';
	variable receive_flag		: std_logic := '0';

begin
	-----------------------------------------------------------------------------------------------
	--if	rising_edge(w_sys_clk) then		-- Denne prosessen m ikke vre klokket.
		---------------------------------------------------------------------------------------
		if	w_reset = '1' then
			-------------------------------------------------------------------------------
			w_IOselect 	<= "00000";
			w_IOwrite	<= '0';
			w_IOread	<= '0';
			w_Data_bus	<= "ZZZZZZZZ";
			last_write_address	:= "00000";	-- Added recently
			last_read_address	:= "00000";	-- Added recently
			transmit_flag		:= '0';		-- Added recently
			receive_flag		:= '0';		-- Added recently
		---------------------------------------------------------------------------------------
		elsif	transmit_flag = '1' then
		
			w_IOselect	<= "00000"; -- skrive ZZ?
			w_IOwrite	<= '0';
			w_Data_bus	<= "ZZZZZZZZ";
			
			transmit_flag := '0';
		---------------------------------------------------------------------------------------
		elsif	receive_flag = '1' then -- overfldig!!	 and w_IOread = '1' then	-- Lagt til w_IOread eksperimentelt.
			-------------------------------------------------------------------------------
			case 	last_read_address is
				-------------------------------------
				when "00001" =>	m1_data_in <= w_Data_bus;
				when "00010" =>	m2_data_in <= w_Data_bus;
				when "00100" =>	m3_data_in <= w_Data_bus;
				when "01000" =>	m4_data_in <= w_Data_bus;
				when "10000" =>	m5_data_in <= w_Data_bus;
				when others => null;
				-------------------------------------
			end case;
			-------------------------------------------------------------------------------
			w_IOselect	<= "00000";
			w_IOread	<= '0';

			receive_flag	:= '0';
		---------------------------------------------------------------------------------------
		elsif	(write_address /= last_write_address) and (write_address /= "00000") then
			
			w_IOselect	<= write_address;
			w_IOwrite	<= '1';
			w_Data_bus	<= data_to_write;

			last_write_address := write_address;

			transmit_flag := '1';
		---------------------------------------------------------------------------------------
		-- Trenger antakelig tx_high_byte ogs, fordi last_adr ellers vil vre lik som low_byte sin ard.
		---------------------------------------------------------------------------------------
		elsif	((read_address /= last_read_address) and (read_address /= "00000")) or tx_high_byte = '1' then -- tx ndvendig?
			
			w_IOselect 	<= read_address;
			w_IOread	<= '1';

			last_read_address := read_address;
			
			receive_flag := '1';
		---------------------------------------------------------------------------------------
--		else	-- Denne skal srge for  frigi bussen etter bruk.
--			w_IOselect	<= "00000";
--			w_IOwrite	<= '0';
--			w_IOread	<= '0';
--			w_Data_bus	<= "ZZZZZZZZ";
		---------------------------------------------------------------------------------------
		end if;
	-----------------------------------------------------------------------------------------------
	--end if;
	
end process;
-- ******************************************************************************************* --



-- ******************************************************************************************* --
-- ******************************************************************************************* --
Interrupt_handeler_proc : process(W_SYS_CLK, w_reset) is	--, w_interrupt_out) is -- Vil ikke at endringer her skal tukle til:, W_INTERRUPT_OUT) is

	variable low_byte_received 	: std_logic_vector(4 downto 0) := "00000";
	variable transfer_initiated 	: std_logic_vector(4 downto 0) := "00000";
	variable transfer_packet_flag	: std_logic_vector(4 downto 0) := "00000";
begin
	-----------------------------------------------------------------------------------------
	if	w_reset = '1' then
		---------------------------------------------------------------------------------
		low_byte_received	:= "00000";
		transfer_initiated	:= "00000";
		transfer_packet_flag	:= "00000";
		tx_high_byte		<= '0';
		read_address		<= "00000";
		
		tb_m1_speed		<= (others => '0');
		tb_m2_speed		<= (others => '0');
		tb_m3_speed		<= (others => '0');
		tb_m4_speed		<= (others => '0');
		tb_m5_speed		<= (others => '0');
	-----------------------------------------------------------------------------------------
	--if w_interrupt_out /= "00000" then
	elsif rising_edge(w_sys_clk) then					--and w_interrupt_out /= "00000" then 	-- Fr aldri high-byte med denne.
		---------------------------------------------------------------------------------
		if 	(w_interrupt_out(m1) = '1' and transfer_packet_flag = "00000") or transfer_packet_flag(m1) = '1' then	-- Lagt til tx_init eksperimentelt.
			-------------------------------------------------------------------------
			if	transfer_initiated(m1) = '0' and low_byte_received(m1) = '0' then
	
				read_address <= m1_adr;		-- Tror dette skal funke!

				transfer_initiated(m1) 	:= '1';
				transfer_packet_flag(m1):= '1';
			-------------------------------------------------------------------------	
			elsif 	transfer_initiated(m1) = '1' and low_byte_received(m1) = '0' then
	
				tb_m1_speed(7 downto 0) <= m1_data_in;
	
				read_address <= "00000";	-- Tror dette m til for  kunne lese fra samme igjen senere.
	
				transfer_initiated(m1)	:= '0';
				low_byte_received(m1)	:= '1';
			-------------------------------------------------------------------------	
			elsif	transfer_initiated(m1) = '0' and low_byte_received(m1) = '1' then
	
				read_address <= m1_adr;

				transfer_initiated(m1) := '1';
				tx_high_byte		<= '1';	-- Dette er kanskje litt tvilsomt. Men br g greit i testbenken.
			-------------------------------------------------------------------------	
			elsif 	transfer_initiated(m1) = '1' and low_byte_received(m1) = '1' then

				tb_m1_speed(15 downto 8) <= m1_data_in;

				read_address <= "00000";	-- Tror dette m til for  kunne lese fra samme igjen senere.

				transfer_initiated(m1)	:= '0';
				low_byte_received(m1)	:= '0';
				transfer_packet_flag(m1):= '0';
				tx_high_byte		<= '0';
			-------------------------------------------------------------------------
			end if;
		---------------------------------------------------------------------------------
		elsif 	(w_interrupt_out(m2) = '1' and transfer_packet_flag = "00000") or transfer_packet_flag(m2) = '1' then	-- Lagt til tx_init eksperimentelt.
			-------------------------------------------------------------------------
			if	transfer_initiated(m2) = '0' and low_byte_received(m2) = '0' then
	
				read_address <= m2_adr;		-- Tror dette skal funke!

				transfer_initiated(m2) 	:= '1';
				transfer_packet_flag(m2):= '1';
			-------------------------------------------------------------------------	
			elsif 	transfer_initiated(m2) = '1' and low_byte_received(m2) = '0' then
	
				tb_m2_speed(7 downto 0) <= m2_data_in;
	
				read_address <= "00000";	-- Tror dette m til for  kunne lese fra samme igjen senere.
	
				transfer_initiated(m2)	:= '0';
				low_byte_received(m2)	:= '1';
			-------------------------------------------------------------------------	
			elsif	transfer_initiated(m2) = '0' and low_byte_received(m2) = '1' then
	
				read_address <= m2_adr;

				transfer_initiated(m2) := '1';
				tx_high_byte		<= '1';	-- Dette er kanskje litt tvilsomt. Men br g greit i testbenken.
			-------------------------------------------------------------------------	
			elsif 	transfer_initiated(m2) = '1' and low_byte_received(m2) = '1' then

				tb_m2_speed(15 downto 8) <= m2_data_in;

				read_address <= "00000";	-- Tror dette m til for  kunne lese fra samme igjen senere.

				transfer_initiated(m2)	:= '0';
				low_byte_received(m2)	:= '0';
				transfer_packet_flag(m2):= '0';
				tx_high_byte		<= '0';
			-------------------------------------------------------------------------
			end if;
		---------------------------------------------------------------------------------
		elsif 	(w_interrupt_out(m3) = '1' and transfer_packet_flag = "00000") or transfer_packet_flag(m3) = '1' then	-- Lagt til tx_init eksperimentelt.
			-------------------------------------------------------------------------
			if	transfer_initiated(m3) = '0' and low_byte_received(m3) = '0' then
	
				read_address <= m3_adr;		-- Tror dette skal funke!

				transfer_initiated(m3) 	:= '1';
				transfer_packet_flag(m3):= '1';
			-------------------------------------------------------------------------	
			elsif 	transfer_initiated(m3) = '1' and low_byte_received(m3) = '0' then
	
				tb_m3_speed(7 downto 0) <= m3_data_in;
	
				read_address <= "00000";	-- Tror dette m til for  kunne lese fra samme igjen senere.
	
				transfer_initiated(m3)	:= '0';
				low_byte_received(m3)	:= '1';
			-------------------------------------------------------------------------	
			elsif	transfer_initiated(m3) = '0' and low_byte_received(m3) = '1' then
	
				read_address <= m3_adr;

				transfer_initiated(m3) := '1';
				tx_high_byte		<= '1';	-- Dette er kanskje litt tvilsomt. Men br g greit i testbenken.
			-------------------------------------------------------------------------	
			elsif 	transfer_initiated(m3) = '1' and low_byte_received(m3) = '1' then

				tb_m3_speed(15 downto 8) <= m3_data_in;

				read_address <= "00000";	-- Tror dette m til for  kunne lese fra samme igjen senere.

				transfer_initiated(m3)	:= '0';
				low_byte_received(m3)	:= '0';
				transfer_packet_flag(m3):= '0';
				tx_high_byte		<= '0';
			-------------------------------------------------------------------------
			end if;
		---------------------------------------------------------------------------------
		elsif 	(w_interrupt_out(m4) = '1' and transfer_packet_flag = "00000") or transfer_packet_flag(m4) = '1' then	-- Lagt til tx_init eksperimentelt.
			-------------------------------------------------------------------------
			if	transfer_initiated(m4) = '0' and low_byte_received(m4) = '0' then
	
				read_address <= m4_adr;		-- Tror dette skal funke!

				transfer_initiated(m4) 	:= '1';
				transfer_packet_flag(m4):= '1';
			-------------------------------------------------------------------------	
			elsif 	transfer_initiated(m4) = '1' and low_byte_received(m4) = '0' then
	
				tb_m4_speed(7 downto 0) <= m4_data_in;
	
				read_address <= "00000";	-- Tror dette m til for  kunne lese fra samme igjen senere.
	
				transfer_initiated(m4)	:= '0';
				low_byte_received(m4)	:= '1';
			-------------------------------------------------------------------------	
			elsif	transfer_initiated(m4) = '0' and low_byte_received(m4) = '1' then
	
				read_address <= m4_adr;

				transfer_initiated(m4) := '1';
				tx_high_byte		<= '1';	-- Dette er kanskje litt tvilsomt. Men br g greit i testbenken.
			-------------------------------------------------------------------------	
			elsif 	transfer_initiated(m4) = '1' and low_byte_received(m4) = '1' then

				tb_m4_speed(15 downto 8) <= m4_data_in;

				read_address <= "00000";	-- Tror dette m til for  kunne lese fra samme igjen senere.

				transfer_initiated(m4)	:= '0';
				low_byte_received(m4)	:= '0';
				transfer_packet_flag(m4):= '0';
				tx_high_byte		<= '0';
			-------------------------------------------------------------------------
			end if;
		---------------------------------------------------------------------------------
		elsif 	(w_interrupt_out(m5) = '1' and transfer_packet_flag = "00000") or transfer_packet_flag(m5) = '1' then	-- Lagt til tx_init eksperimentelt.
			-------------------------------------------------------------------------
			if	transfer_initiated(m5) = '0' and low_byte_received(m5) = '0' then
	
				read_address <= m5_adr;		-- Tror dette skal funke!

				transfer_initiated(m5) 	:= '1';
				transfer_packet_flag(m5):= '1';
			-------------------------------------------------------------------------	
			elsif 	transfer_initiated(m5) = '1' and low_byte_received(m5) = '0' then
	
				tb_m5_speed(7 downto 0) <= m5_data_in;
	
				read_address <= "00000";	-- Tror dette m til for  kunne lese fra samme igjen senere.
	
				transfer_initiated(m5)	:= '0';
				low_byte_received(m5)	:= '1';
			-------------------------------------------------------------------------	
			elsif	transfer_initiated(m5) = '0' and low_byte_received(m5) = '1' then
	
				read_address <= m5_adr;

				transfer_initiated(m5) := '1';
				tx_high_byte		<= '1';	-- Dette er kanskje litt tvilsomt. Men br g greit i testbenken.
			-------------------------------------------------------------------------	
			elsif 	transfer_initiated(m5) = '1' and low_byte_received(m5) = '1' then

				tb_m5_speed(15 downto 8) <= m5_data_in;

				read_address <= "00000";	-- Tror dette m til for  kunne lese fra samme igjen senere.

				transfer_initiated(m5)	:= '0';
				low_byte_received(m5)	:= '0';
				transfer_packet_flag(m5):= '0';
				tx_high_byte		<= '0';
			-------------------------------------------------------------------------
			end if;
		end if;
	-----------------------------------------------------------------------------------------
	end if;

end process;
-- ******************************************************************************************* --

	--constant MOTOR_FQ_100k	: time := 100 us;	-- Gir 100.000 RPM, eller 1666,67 RPS.
	--constant MOTOR_FQ_10k	: time := 1 ms;		-- Gir 10.000 RPM, eller 166,67 RPS.
	--constant MOTOR_FQ_1k	: time := 10 ms;	-- Gir 1.000 RPM, eller 16,67 RPS.
	--constant MOTOR_FQ_24k	: time := 416666 ns;	-- Gir 24.000 RPM. (400 RPS)
	--constant MOTOR_FQ_08k	: time := 12500000 ns;	-- Gir 800 RPM. 

-- ******************************************************************** --
-- ******************************************************************** --
stimuli_m1 : process
begin

--   	w_m1_hall_sensors   <= (others => '0');

   	while (true) loop
   
   		w_m1_hall_sensors <= "100"; wait for MOTOR_FQ_10k;
   		w_m1_hall_sensors <= "110"; wait for MOTOR_FQ_10k;
   		w_m1_hall_sensors <= "010"; wait for MOTOR_FQ_10k;
   		w_m1_hall_sensors <= "011"; wait for MOTOR_FQ_10k;
   		w_m1_hall_sensors <= "001"; wait for MOTOR_FQ_10k;
   		w_m1_hall_sensors <= "101"; wait for MOTOR_FQ_10k;
   		
   	end loop;
   	
   	wait;
end process;
-- ******************************************************************** --

-- ******************************************************** --
stimuli_m2 : process
begin

   	w_m2_hall_sensors   <= (others => '0');

   	while (true) loop

   		w_m2_hall_sensors <= "100"; wait for MOTOR_FQ_24k;
   		w_m2_hall_sensors <= "110"; wait for MOTOR_FQ_24k;
   		w_m2_hall_sensors <= "010"; wait for MOTOR_FQ_24k;
   		w_m2_hall_sensors <= "011"; wait for MOTOR_FQ_24k;
   		w_m2_hall_sensors <= "001"; wait for MOTOR_FQ_24k;
   		w_m2_hall_sensors <= "101"; wait for MOTOR_FQ_24k;
   		
   	end loop;
	
	wait;
end process;
-- ******************************************************** --

-- ******************************************************** --
--stimuli_m3 : process
--begin
--
-- --  	w_m3_hall_sensors   <= (others => '0');
--
--   	while (true) loop
--
--   		w_m3_hall_sensors <= "100"; wait for MOTOR_FQ_10k;
--   		w_m3_hall_sensors <= "110"; wait for MOTOR_FQ_10k;
--   		w_m3_hall_sensors <= "010"; wait for MOTOR_FQ_10k;
--   		w_m3_hall_sensors <= "011"; wait for MOTOR_FQ_10k;
--   		w_m3_hall_sensors <= "001"; wait for MOTOR_FQ_10k;
--   		w_m3_hall_sensors <= "101"; wait for MOTOR_FQ_10k;
--   		
--   	end loop;
--	
--	wait;
--end process;
-- ******************************************************** --

-- ******************************************************** --
--stimuli_m4 : process
--begin
--
-- --  	w_m4_hall_sensors   <= (others => '0');
--
--   	while (true) loop
--
--   		w_m4_hall_sensors <= "100"; wait for MOTOR_FQ_10k;
--   		w_m4_hall_sensors <= "110"; wait for MOTOR_FQ_10k;
--   		w_m4_hall_sensors <= "010"; wait for MOTOR_FQ_10k;
--   		w_m4_hall_sensors <= "011"; wait for MOTOR_FQ_10k;
--   		w_m4_hall_sensors <= "001"; wait for MOTOR_FQ_10k;
--   		w_m4_hall_sensors <= "101"; wait for MOTOR_FQ_10k;
--   		
--   	end loop;
--	
--	wait;
--end process;
-- ******************************************************** --

-- ******************************************************** --
--stimuli_m5 : process
--begin
--
-- 	--w_m5_hall_sensors   <= (others => '0');
--
--   	while (true) loop
--
--   		w_m5_hall_sensors <= "100"; wait for MOTOR_FQ_10k;
--   		w_m5_hall_sensors <= "110"; wait for MOTOR_FQ_10k;
--   		w_m5_hall_sensors <= "010"; wait for MOTOR_FQ_10k;
--   		w_m5_hall_sensors <= "011"; wait for MOTOR_FQ_10k;
--   		w_m5_hall_sensors <= "001"; wait for MOTOR_FQ_10k;
--   		w_m5_hall_sensors <= "101"; wait for MOTOR_FQ_10k;
--   		
--   	end loop;
--	
--	wait;
--end process;
-- ******************************************************** --

end architecture;
-- ************************************************************************************************************************************* --